#include <time.h>
#include "muh_blood.h"
#include <iostream>
#include "cpudm.h"
#include <cmath>
#include "melanin.h"
using namespace std;

//A value from n = 1.4 for tissue, 1.0 for the surrounding air

#define div13 1.0f/3.0f
#define A 0.14386f
//#define A 0.17f
#define de 100e-6*1.0f

double Rsp = pow((1.0f-1.4f)/(1.0f+1.4f),2);


//takes in the arrays and the next line, which will only be transferred and processed in the next round

//isotropic source function, three-layer model
void ReflIsoL3(float mua1, float mus1, float mua2, float mus2, float mua3, float mus3, float gcol, float d1, float d2, float *res){
	float g = gcol;

	//reduced scattering coefficients
	float musr1 = mus1*(1.0f-g);
	float musr2 = mus2*(1.0f-g);
	float musr3 = mus3*(1.0f-g);


	//diffusion constant
	float D1 = 1.0f/(3.0f*(musr1 + mua1));
	float D2 = 1.0f/(3.0f*(musr2 + mua2));
	float D3 = 1.0f/(3.0f*(musr3 + mua3));
	
	//transport coefficients
	float mutr1 = musr1 + mua1;
	float u1 = mutr1;
	float mutr2 = musr2 + mua2;
	float u2 = mutr2;
	float mutr3 = musr3 + mua3;
	float u3 = mutr3;

	//optical penetration depth
	float del1 = sqrt(D1/(mua1));
	float a1 = del1;
	float del2 = sqrt(D2/(mua2));
	float a2 = del2;
	float del3 = sqrt(D3/(mua3));
	float a3 = del3;

	float K1 = del1*del1*musr1/(D1*(1-mutr1*mutr1*del1*del1));
	float K2 = del2*del2*musr2/(D2*(1-mutr2*mutr2*del2*del2));
	float K3 = del3*del3*musr3/(D3*(1-mutr3*mutr3*del3*del3));
	float A2 = -(-2*D2*exp(-u1*d1)*exp(u2*(-d2+d1))*(K3-K2)*D1*D3*a2-D1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*a2*D3*K2+D1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*a2*D3*K1+D1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*a2*D3*K1-D1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*a2*D3*K2-exp((-d2+d1)/a2)*D3*a2*a2*D1*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)+exp(-(-d2+d1)/a2)*D3*a2*a2*D1*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)+D1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*D2*a3*K2-D1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*D2*a3*K1-D1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*D2*a3*K2+D1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*D2*a3*K1-2*A*a1*D2*exp(-u1*d1)*exp(u2*(-d2+d1))*(K3-K2)*D3*a2-2*exp(-u1*d1-u2*d2+u2*d1)*a2*D2*D2*u2*K2*a3*D1+A*a1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*a2*D3*K1-A*a1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*a2*D3*K2+A*a1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*a2*D3*K1-A*a1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*a2*D3*K2+A*a1*exp(-(-d2+d1)/a2)*a3*D2*a2*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)+A*a1*exp((-d2+d1)/a2)*D2*a3*a2*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)+2*exp(-u1*d1-u2*d2+u2*d1)*a2*D3*K3*u3*a3*D2*D1+A*a1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*D2*a3*K2-A*a1*exp(-(u1*d1*a2-d1+d2)/a2)*D2*D2*a3*K1+A*a1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*D2*a3*K1-A*a1*exp(-(-d2+d1+u1*d1*a2)/a2)*D2*D2*a3*K2+exp((-d2+d1)/a2)*D2*a3*a2*D1*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)+exp(-(-d2+d1)/a2)*a3*D2*a2*D1*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)-A*a1*exp((-d2+d1)/a2)*D3*a2*a2*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)+A*a1*exp(-(-d2+d1)/a2)*D3*a2*a2*exp(-u1*d1)*(D2*u2*K2-D1*u1*K1)+2*A*a1*exp(-u1*d1-u2*d2+u2*d1)*a2*D3*K3*u3*a3*D2-2*A*a1*exp(-u1*d1-u2*d2+u2*d1)*a2*D2*D2*u2*K2*a3-A*a1*a2*D2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*D3-A*a1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*a2*D2*K1*D3+D1*D1*a2*D2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*u1*a3+D1*a1*D2*D2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*u1*a3+A*D1*a2*D2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*a3+D1*D1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*a2*D2*K1*u1*a3+A*D1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*a2*D2*K1*a3-D1*a1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*D2*D2*K1*u1*a3+A*D1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*a2*a2*K1*D3+D1*D1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*a2*a2*K1*u1*D3-D1*D1*a2*a2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*u1*D3-A*D1*a2*a2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*D3+A*a1*D2*D2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*a3-A*a1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*D2*D2*K1*a3-D1*a1*a2*D2*exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*K1*u1*D3-D1*a1*exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*a2*D2*K1*u1*D3)*a1/(exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*a1*D2*D2*a3+exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*D1*a2*D2*a3+exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*a1*D2*D2*a3+exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*D1*a1*D2*D2*a3-exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*D1*D1*a2*D2*a3-exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*a1*D2*D2*a3+exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*a1*D2*D2*a3-exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*D1*a2*D2*a3-exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*a1*D2*D2*a3-exp((d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*a1*D2*D2*a3+exp((d1*a2-d2*a1+d1*a1)/a1/a2)*D1*D1*a2*D2*a3-exp((d1*a2-d2*a1+d1*a1)/a1/a2)*D1*a1*D2*D2*a3+exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*a1*D2*a2*D3+exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*a1*D2*a2*D3+exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*D1*a2*a2*D3+exp((d1*a2-d2*a1+d1*a1)/a1/a2)*D1*a1*D2*a2*D3+exp((d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*a1*D2*a2*D3-exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*a1*D2*a2*D3+exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*D1*a2*a2*D3-exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*a1*D2*a2*D3-exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*D1*a2*a2*D3+exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*D1*a1*D2*a2*D3-exp((d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*D1*a2*a2*D3+exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*a1*D2*a2*D3+exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*D1*a2*D2*a3+exp((d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*D1*a2*D2*a3+exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*A*a1*D1*a2*D2*a3+exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*A*a1*D1*a2*D2*a3+exp(-(-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*D1*a2*a2*D3-exp((d1*a2-d2*a1+d1*a1)/a1/a2)*D1*D1*a2*a2*D3-exp(-(d1*a2-d2*a1+d1*a1)/a1/a2)*D1*D1*a2*a2*D3+exp((-d1*a2+d1*a1-d2*a1)/a1/a2)*D1*D1*a2*a2*D3);
	float A1 = (A2*(D1/del1-A)-K1*(A+D1*mutr1))/(D1/del1+A);
	float j = K1*D1*mutr1 + D1/del1*A1 - A2*D1/del1;
	*res = -j;
//	*res = *res*(1-Rsp);
}

//takes in the skin optical properties and outputs the reflectance to res, and the derivative of the reflectance with respect to muad to deriv
//isotropic source function, two-layer model
void ReflIsoL2(float muae, float muse, float muad, float musd, float gcol, float *res, float *deriv){
	float derivmuae;
	ReflIsoL2(muae, muse, muad, musd, gcol, res, deriv, &derivmuae);
}
void ReflIsoL2(float muae, float muse, float muad, float musd, float gcol, float *res, float *derivmuad, float *derivmuae){
	float g = gcol;

	//reduced scattering coefficients
	float musr1 = muse*(1.0f-g);
	float musr2 = musd*(1.0f-g);

	//move mua into shared memory
	float curr_muae = muae;
	float curr_muad = muad;
	
	//diffusion constant
	float D1 = 1.0f/(3.0f*(musr1 + curr_muae));
	float D2 = 1.0f/(3.0f*(musr2 + curr_muad));

	float musr2dmusr1 = musr2/(musr1); //musr2 divided by musr1, keep for derivative calc


	//optical penetration depth
	float del1 = sqrt(D1/(curr_muae));
	float del2 = sqrt(D2/(curr_muad));

	//from Svaasand 1995	
	//calculate the reflectance value
	float sinhval = sinh(de/( del1));
	float coshval = cosh(de/( del1));
	float fact1 = del1*musr1*A;
	float f1 = (del1*del1*del2*div13-del1*del1*D2)*coshval+(del1*del1*del1*D2/D1*div13 - del1*del2*D1)*sinhval; //keeping for derivative calc
	float f2 = 1.0f+del2/D2*div13; //keeping for derivative calc
	float expval = exp(-de/D1*div13);
	float f3 = (musr2dmusr1*del2*del2*D1*(del1*del1/(D1*D1)*div13*div13-1.0f)+del1*del1*(D2-del2*del2/D2*div13*div13))*expval; //keeping for derivative calc
	float f4 = del1*del1/(D1*D1)*div13*div13 - 1.0f; //keep for derivative calc, f4
	float f5 = del2/D2*div13+1.0f; //keep for derivative calc, f5
	float f6 = D1*del1*(D2+del2*A)*coshval+(D1*D1*del2 + D2*del1*del1*A)*sinhval; //keep for derivative calc, f6
	float num = fact1*(f1*f2+f3);
	float denom = f4*f5*f6;
	*res = num/denom;

	//DERMISDERIVERT
	//calculate the derivative with respect to muad
	float dD2dmuad = -3.0f*D2*D2;
	float ddel2dmuad = (dD2dmuad*curr_muad-D2)*1.0f/(curr_muad*curr_muad)*1.0f/(2.0f*del2);
	float df2dmuad = div13*(ddel2dmuad*D2 - del2*dD2dmuad)*1.0f/(D2*D2);
	*derivmuad = (fact1*((coshval*(del1*del1*div13*ddel2dmuad - del1*del1*dD2dmuad) + sinhval*(del1*del1*del1*1.0f/D1*div13*dD2dmuad - del1*D1*ddel2dmuad))*f2 + f1*df2dmuad + expval*(musr2dmusr1*2.0f*del2*ddel2dmuad*D1*((del1*del1)/(9.0f*D1*D1)-1.0f) + del1*del1*(dD2dmuad + 1.0f/(9.0f*curr_muad*curr_muad))))*denom - (f4*(df2dmuad*f6 + D1*del1*(dD2dmuad + ddel2dmuad*A)*coshval + (D1*D1*ddel2dmuad + dD2dmuad*del1*del1*A)*sinhval*f5)*num))*1.0f/(denom*denom);
	

	float dD1dmuae = -3.0f*D1*D1;
	float ddel1dmuae = (dD1dmuae*curr_muae-D1)*1.0f/(curr_muae*curr_muae)*1.0f/(2.0f*del1);
	float d1 = de;
	float derivsinhval = -coshval*de/(del1*del1)*ddel1dmuae;
	float derivcoshval = -sinhval*de/(del1*del1)*ddel1dmuae;
	
	//EPIDERMISDERIVERT
	//omrokkering for å få ting til å passe med notasjon for epidermisderivert
	f1 = (del1*del1*del2/3.0f-del1*del1*D2)*coshval + (del1*del1*del1*D2/(3.0f*D1)-del1*del2*D1)*sinhval;
	f3 = (musr2dmusr1*del2*del2*D1*(del1*del1/(9.0f*D1*D1)-1.0f)+del1*del1*(D2-del2*del2/(9.0f*D2)))*expval;
	f4 = del1*del1/(9.0f*D1*D1)-1.0f;
	f2 = del2/(3*D2)+1.0f;
	f5 = D1*del1*(D2+del2*A)*coshval + (D1*D1*del2+D2*del1*del1*A)*sinhval;

	float df1dmuae = (del1*del1*del2/3.0f - del1*del1*D2)*derivcoshval + coshval*2*del1*ddel1dmuae*(del2/3.0f-D2) + (del1*del1*del1*D2/(3.0f*D1) - del1*del2*D1)*derivsinhval + sinhval*((3*del1*del1*ddel1dmuae*D2*D1 - del1*del1*del1*D2*dD1dmuae)/(3.0f*D1*D1) - del2*ddel1dmuae*D1 - del2*del1*dD1dmuae);
	float df3dmuae = expval*(musr2dmusr1*del2*del2*(dD1dmuae*(del1*del1/(9.0f*D1*D1)-1.0f) + (2.0f*del1*ddel1dmuae*D1 - del1*del1*2.0f*dD1dmuae)/(9.0f*D1*D1)) + 2.0f*del1*ddel1dmuae*(D2-del2*del2/(9.0f*D2))) + f3*(d1/(9.0f*D1*D1)*dD1dmuae);
	float df4dmuae = (2.0f*del1*ddel1dmuae*D1*D1 - del1*del1*2.0f*D1*dD1dmuae)/(9.0f*D1*D1*D1*D1);
	float df5dmuae = (D2+del2*A)*coshval*(del1*dD1dmuae + D1*ddel1dmuae) + D1*del1*(D2+del2*A)*derivcoshval + sinhval*(2.0f*D1*dD1dmuae*del2 + D2*2.0f*del1*ddel1dmuae*A) + (D1*D1*del2 + D2*del1*del1*A)*derivsinhval;

	float mua1 = muae;
	float mua2 = muad;

	/* OPPSTYKKING I SNGULARITETSDEL OG IKKE-SINGULARITETSDEL
	float del1D1 = sqrt(3*(musr1/mua1 + 1));
	float D1del1 = sqrt(mua1/(3*(musr1+mua1)));
	float del2D2 = sqrt(3*(musr2/mua2 + 1));
	float D2del2 = sqrt(mua2/(3*(musr2+mua2)));

	float nevner = (del1D1/3+1)*(del2D2/3+1)*((A+D1del1)*(D2del2+D1del1)*exp(d1/del1)+(A-D1/del1)*(D1del1-D2del2)*exp(-d1/del1)); //frikjent
	float fellesteller = del1*musr1*A;
	float snill = (D1del1+D2del2)*exp(d1/del1)*(1+del2D2/3) + 2*musr2dmusr1*del2*mua1*(del1D1/3+1)*exp(-d1/(3*D1)); //frikjent
	float faenskap1 = (-(del1/(3*D1)+1)*exp(-d1/del1)+2*exp(-d1/(3*D1)))/(del1/(3*D1)-1);
	float faenskap2 = ((del1/(3*D1)+1)*D1/del1*exp(-d1/del1)-2.0f/3.0f*exp(-d1/(3*D1)))/(del1/(3*D1)-1);
	float slem = (1+del2/(3*D2))*(D2/del2*faenskap1 + faenskap2);
	*derivmuae = fellesteller*(snill + slem)/nevner; */

	*derivmuae = sqrt(0.3e1) * A * ((-(D2 * D2 * (0.1e1 / 0.3e1 + A) * pow(musr1, 0.3e1) / 0.4e1 + 0.3e1 / 0.4e1 * mua1 * ((-0.5e1 / 0.9e1 + A) * D2 * D2 - 0.8e1 / 0.9e1 * D2 * del2 * A + del2 * del2 * (0.1e1 / 0.3e1 + A) / 0.9e1) * musr1 * musr1 + 0.3e1 / 0.2e1 * mua1 * mua1 * ((-0.5e1 / 0.9e1 + A) * D2 * D2 - 0.8e1 / 0.9e1 * del2 * (A - 0.1e1 / 0.2e1) * D2 + del2 * del2 * (-0.1e1 + A) / 0.9e1) * musr1 + pow(mua1, 0.3e1) * ((A - 0.1e1 / 0.3e1) * D2 * D2 - 0.2e1 / 0.3e1 * del2 * (-0.1e1 + A) * D2 + (A - 0.1e1 / 0.3e1) * del2 * del2 / 0.3e1)) * (D2 + del2 / 0.3e1) * musr1 * cosh(d1 * sqrt(0.3e1) * sqrt(musr1 + mua1) * sqrt(mua1)) + exp(-d1 * (musr1 + mua1)) * (-((mua1 * (0.1e1 / 0.2e1 + A) * d1 - A / 0.2e1) * D2 + mua1 * d1 * del2 * A / 0.2e1) * (-del2 * del2 / 0.9e1 + D2 * D2 + musr2 * del2 * del2 * D2 / 0.3e1) * pow(musr1, 0.4e1) / 0.2e1 + 0.3e1 / 0.4e1 * mua1 * ((-d1 * mua1 / 0.3e1 + A) * pow(D2, 0.3e1) + 0.4e1 / 0.9e1 * ((mua1 * (0.1e1 / 0.4e1 + A) * d1 - 0.3e1 / 0.4e1 * A) * musr2 * del2 - 0.1e1 / 0.4e1 - 0.3e1 / 0.4e1 * mua1 * (A + 0.2e1 / 0.3e1) * d1) * del2 * D2 * D2 + (musr2 * (-0.1e1 / 0.3e1 + mua1 * (A - 0.2e1 / 0.3e1) * d1) * del2 - A + d1 * mua1 / 0.3e1) * del2 * del2 * D2 / 0.9e1 + (0.1e1 / 0.3e1 + mua1 * (A + 0.2e1 / 0.3e1) * d1) * pow(del2, 0.3e1) / 0.27e2) * pow(musr1, 0.3e1) + 0.3e1 / 0.2e1 * mua1 * mua1 * ((mua1 * (A + 0.2e1 / 0.3e1) * d1 + A) * pow(D2, 0.3e1) + (mua1 * musr2 * d1 * (0.1e1 + A) * del2 + 0.1e1 + 0.2e1 * mua1 * (0.1e1 / 0.6e1 + A) * d1) * del2 * D2 * D2 / 0.3e1 + ((0.2e1 / 0.9e1 + mua1 * (0.1e1 / 0.3e1 + A) * d1) * musr2 * del2 - mua1 * (A + 0.2e1 / 0.3e1) * d1 / 0.3e1 - A / 0.3e1) * del2 * del2 * D2 / 0.3e1 - 0.2e1 / 0.27e2 * pow(del2, 0.3e1) * (0.1e1 / 0.2e1 + mua1 * (0.1e1 / 0.6e1 + A) * d1)) * musr1 * musr1 + pow(mua1, 0.3e1) * ((mua1 * (0.1e1 + A) * d1 + A) * pow(D2, 0.3e1) - 0.2e1 / 0.3e1 * ((mua1 * (0.1e1 / 0.2e1 + A) * d1 - A / 0.2e1) * musr2 * del2 - 0.1e1 / 0.2e1 - 0.3e1 / 0.2e1 * mua1 * (0.1e1 / 0.3e1 + A) * d1) * del2 * D2 * D2 - del2 * del2 * ((mua1 * d1 * A + 0.1e1 / 0.3e1) * musr2 * del2 + mua1 * (0.1e1 + A) * d1 / 0.3e1 + A / 0.3e1) * D2 / 0.3e1 - pow(del2, 0.3e1) * (0.1e1 / 0.3e1 + mua1 * (0.1e1 / 0.3e1 + A) * d1) / 0.9e1) * musr1 - 0.2e1 / 0.3e1 * d1 * pow(mua1, 0.5e1) * ((0.1e1 + A) * D2 + del2 * (0.1e1 / 0.3e1 + A)) * del2 * del2 * musr2 * D2)) * sinh(d1 * sqrt(0.3e1) * sqrt(musr1 + mua1) * sqrt(mua1)) - sqrt(0.3e1) * (-0.4e1 / 0.3e1 * (D2 + del2 / 0.3e1) * (0.2e1 * musr1 * ((A - 0.1e1 / 0.2e1) * D2 * D2 - del2 * (-0.1e1 + A) * D2 / 0.2e1 + del2 * del2 * A / 0.6e1) * pow(mua1, 0.3e1 / 0.2e1) + ((-0.1e1 + A) * D2 * D2 - 0.2e1 * (A - 0.1e1 / 0.3e1) * del2 * D2 + del2 * del2 * (-0.1e1 + A) / 0.3e1) * pow(mua1, 0.5e1 / 0.2e1) + (D2 * A - del2 * (0.1e1 / 0.3e1 + A) / 0.2e1) * sqrt(mua1) * musr1 * musr1 * D2) * musr1 * pow(cosh(d1 * sqrt(0.3e1) * sqrt(musr1 + mua1) * sqrt(mua1)), 0.2e1) + exp(-d1 * (musr1 + mua1)) * ((d1 * ((A + 0.2e1 / 0.3e1) * pow(D2, 0.3e1) - ((A - 0.2e1 / 0.3e1) * musr2 * del2 - 0.1e1 - 0.2e1 * A) * del2 * D2 * D2 / 0.3e1 + 0.2e1 / 0.9e1 * del2 * del2 * ((0.1e1 / 0.2e1 + A) * musr2 * del2 - A / 0.2e1 - 0.1e1 / 0.3e1) * D2 - 0.2e1 / 0.27e2 * (0.1e1 / 0.2e1 + A) * pow(del2, 0.3e1)) * musr1 - 0.4e1 / 0.3e1 * (D2 + del2 / 0.3e1) * (D2 - del2 / 0.3e1) * (D2 + del2 * A)) * musr1 * musr1 * pow(mua1, 0.3e1 / 0.2e1) - 0.4e1 * (D2 + del2 / 0.3e1) * (d1 * ((0.1e1 / 0.6e1 + A) * D2 * D2 + (-0.1e1 / 0.3e1 + musr2 * del2) * del2 * (0.1e1 / 0.3e1 + A) * D2 / 0.2e1 - del2 * del2 * A / 0.18e2) * musr1 + (D2 - del2 / 0.3e1) * (D2 + del2 * A) / 0.3e1) * musr1 * pow(mua1, 0.5e1 / 0.2e1) + d1 * (-0.4e1 * musr1 * ((0.1e1 / 0.3e1 + A) * pow(D2, 0.3e1) - del2 * (del2 * musr2 * A - 0.1e1 - A) * D2 * D2 / 0.3e1 - del2 * del2 * (-musr2 * del2 + 0.1e1 / 0.3e1 + A) * D2 / 0.9e1 - pow(del2, 0.3e1) * (0.1e1 + A) / 0.27e2) * pow(mua1, 0.7e1 / 0.2e1) + (0.8e1 / 0.3e1 * ((0.1e1 / 0.3e1 + A) * D2 + (0.1e1 + A) * del2 / 0.3e1) * del2 * del2 * musr2 * pow(mua1, 0.9e1 / 0.2e1) + A * (-del2 * del2 / 0.9e1 + D2 * D2 + musr2 * del2 * del2 * D2 / 0.3e1) * sqrt(mua1) * pow(musr1, 0.4e1)) * D2)) * cosh(d1 * sqrt(0.3e1) * sqrt(musr1 + mua1) * sqrt(mua1)) - (D2 + del2 / 0.3e1) * musr1 * ((d1 * (D2 * D2 - del2 * del2 / 0.3e1) * (0.1e1 / 0.3e1 + A) * musr1 - 0.8e1 / 0.3e1 * D2 * (del2 / 0.3e1 + D2 * A)) * musr1 * pow(mua1, 0.3e1 / 0.2e1) + (-0.4e1 * d1 * D2 * D2 * (0.1e1 / 0.3e1 + A) * musr1 - 0.4e1 / 0.3e1 * (D2 - del2) * (del2 / 0.3e1 + D2 * A)) * pow(mua1, 0.5e1 / 0.2e1) - 0.4e1 * d1 * (D2 * D2 - del2 * del2 / 0.3e1) * (0.1e1 / 0.3e1 + A) * pow(mua1, 0.7e1 / 0.2e1) + sqrt(mua1) * (d1 * D2 * (0.1e1 / 0.3e1 + A) * musr1 - 0.4e1 / 0.3e1 * D2 * A + 0.2e1 / 0.3e1 * del2 * (0.1e1 / 0.3e1 + A)) * musr1 * musr1 * D2)) * sqrt(musr1 + mua1) / 0.4e1) * pow(musr1 + mua1, -0.1e1 / 0.2e1) * pow(mua1, -0.1e1 / 0.2e1) * pow(-musr1 / 0.2e1 + mua1, -0.2e1) / (D2 + del2 / 0.3e1) * pow((D2 * A * musr1 + mua1 * (del2 / 0.3e1 + D2 * A)) * sinh(d1 * sqrt(0.3e1) * sqrt(musr1 + mua1) * sqrt(mua1)) + sqrt(0.3e1) * sqrt(musr1 + mua1) * sqrt(mua1) * cosh(d1 * sqrt(0.3e1) * sqrt(musr1 + mua1) * sqrt(mua1)) * (D2 + del2 * A) / 0.3e1, -0.2e1) / 0.2e1; //direkte fra maple

	// GAMMEL DERIVERT MED SINGULARITETSDRITT
	//cout << mua1 << " " << *res << " " << *derivmuae << " ";
	//*derivmuae = (f4*f5*((1+del2/(3*D2))*df1dmuae + df3dmuae) - ((1+del2/(3*D2))*f1+f3)*(df4dmuae*f5 + df5dmuae*f4))/(f4*f4*f5*f5*(1+del2/(3*D2)))*del1*musr1*A + *res/(del1)*ddel1dmuae;
	//cout << *derivmuae << endl;



	//forsøk på noe litt mindre
	*res = D1 * A * del1 * ((((-musr1 * pow(D2, 0.2e1) + musr1 * pow(del2, 0.2e1) / 0.9e1) * D1 - musr2 * pow(del2, 0.2e1) * D2 / 0.9e1) * pow(del1, 0.2e1) + musr2 * pow(del2, 0.2e1) * D2 * pow(D1, 0.2e1)) * exp(-d1 / D1 / 0.3e1) + (D2 + del2 / 0.3e1) * del1 * musr1 * ((D2 - del2 / 0.3e1) * del1 * D1 * cosh(d1 / del1) + sinh(d1 / del1) * (-pow(del1, 0.2e1) * D2 / 0.3e1 + del2 * pow(D1, 0.2e1)))) / (D1 * del1 * (D2 + del2 * A) * cosh(d1 / del1) + (pow(D1, 0.2e1) * del2 + D2 * pow(del1, 0.2e1) * A) * sinh(d1 / del1)) / (D1 - del1 / 0.3e1) / (D2 + del2 / 0.3e1) / (D1 + del1 / 0.3e1);


	float mutr1 = musr1 + mua1;
	float mutr2 = musr2 + mua2;
	float ddel2d2 = -sqrt(0.3e1) * (1 * mua2 + mutr2) * pow(0.1e1 / mutr2 / mua2, -0.1e1 / 0.2e1) * pow(mutr2, -0.2e1) * pow(mua2, -0.2e1) / 0.6e1;
	float ddel1d1 = -sqrt(0.3e1) * (1 * mua1 + mutr1) * pow(0.1e1 / mutr1 / mua1, -0.1e1 / 0.2e1) * pow(mutr1, -0.2e1) * pow(mua1, -0.2e1) / 0.6e1;
	float dD2d2 = -pow(mutr2, -0.2e1) * 1 / 0.3e1;
	float dD1d1 = -pow(mutr1, -0.2e1) * 1 / 0.3e1;

	*derivmuae = ddel1d1 * musr1 * A * (((pow(del1, 0.2e1) * del2 / 0.3e1 - pow(del1, 0.2e1) * D2) * cosh(d1 / del1) + (pow(del1, 0.3e1) * D2 / D1 / 0.3e1 - del1 * del2 * D1) * sinh(d1 / del1)) * (0.1e1 + del2 / D2 / 0.3e1) + (musr2 / musr1 * pow(del2, 0.2e1) * D1 * (pow(del1, 0.2e1) * pow(D1, -0.2e1) / 0.9e1 - 0.1e1) + pow(del1, 0.2e1) * (D2 - pow(del2, 0.2e1) / D2 / 0.9e1)) * exp(-d1 / D1 / 0.3e1)) / (pow(del1, 0.2e1) * pow(D1, -0.2e1) / 0.9e1 - 0.1e1) / (0.1e1 + del2 / D2 / 0.3e1) / (D1 * del1 * (D2 + del2 * A) * cosh(d1 / del1) + (pow(D1, 0.2e1) * del2 + D2 * pow(del1, 0.2e1) * A) * sinh(d1 / del1)) - 0.1e1 / D1 * A * (del1 * (0.6e1 * pow(D1, 0.2e1) * del1 * ((musr1 * pow(D2, 0.2e1) - musr1 * pow(del2, 0.2e1) / 0.9e1) * D1 + musr2 * pow(del2, 0.2e1) * D2 / 0.9e1) * ddel1d1 + dD1d1 * ((((-musr2 * D2 / 0.3e1 - d1 * musr1 / 0.9e1) * pow(del2, 0.2e1) + d1 * musr1 * pow(D2, 0.2e1)) * D1 + musr2 * D2 * d1 * pow(del2, 0.2e1) / 0.9e1) * pow(del1, 0.2e1) - musr2 * pow(del2, 0.2e1) * D2 * pow(D1, 0.2e1) * (0.3e1 * D1 + d1))) * exp(-d1 / D1 / 0.3e1) / 0.3e1 + (ddel1d1 * D1 * (((0.2e1 / 0.3e1 * del2 - 0.2e1 * D2) * D1 - d1 * D2 / 0.3e1) * pow(del1, 0.2e1) + d1 * del2 * pow(D1, 0.2e1)) * cosh(d1 / del1) + sinh(d1 / del1) * (D1 * (pow(del1, 0.2e1) * D2 + (-del2 * D1 + (D2 - del2 / 0.3e1) * d1) * D1) * ddel1d1 - dD1d1 * del1 * (pow(del1, 0.2e1) * D2 / 0.3e1 + pow(D1, 0.2e1) * del2)) * del1) * (D2 + del2 / 0.3e1) * D1 * musr1) / (D1 + del1 / 0.3e1) / (D2 + del2 / 0.3e1) / (D1 * del1 * (D2 + del2 * A) * cosh(d1 / del1) + (pow(D1, 0.2e1) * del2 + D2 * pow(del1, 0.2e1) * A) * sinh(d1 / del1)) / (D1 - del1 / 0.3e1) + 0.2e1 / 0.9e1 * pow(del1, 0.2e1) * ((((-musr1 * pow(D2, 0.2e1) + musr1 * pow(del2, 0.2e1) / 0.9e1) * D1 - musr2 * pow(del2, 0.2e1) * D2 / 0.9e1) * pow(del1, 0.2e1) + musr2 * pow(del2, 0.2e1) * D2 * pow(D1, 0.2e1)) * exp(-d1 / D1 / 0.3e1) + (D2 + del2 / 0.3e1) * del1 * musr1 * (D1 * del1 * (D2 - del2 / 0.3e1) * cosh(d1 / del1) + sinh(d1 / del1) * (-pow(del1, 0.2e1) * D2 / 0.3e1 + pow(D1, 0.2e1) * del2))) * A * (D1 * ddel1d1 - del1 * dD1d1) * pow(D1 + del1 / 0.3e1, -0.2e1) / (D2 + del2 / 0.3e1) / (D1 * del1 * (D2 + del2 * A) * cosh(d1 / del1) + (pow(D1, 0.2e1) * del2 + D2 * pow(del1, 0.2e1) * A) * sinh(d1 / del1)) * pow(D1 - del1 / 0.3e1, -0.2e1) - del1 * musr1 * A * (((pow(del1, 0.2e1) * del2 / 0.3e1 - pow(del1, 0.2e1) * D2) * cosh(d1 / del1) + (pow(del1, 0.3e1) * D2 / D1 / 0.3e1 - del1 * del2 * D1) * sinh(d1 / del1)) * (0.1e1 + del2 / D2 / 0.3e1) + (musr2 / musr1 * pow(del2, 0.2e1) * D1 * (pow(del1, 0.2e1) * pow(D1, -0.2e1) / 0.9e1 - 0.1e1) + pow(del1, 0.2e1) * (D2 - pow(del2, 0.2e1) / D2 / 0.9e1)) * exp(-d1 / D1 / 0.3e1)) / (pow(del1, 0.2e1) * pow(D1, -0.2e1) / 0.9e1 - 0.1e1) / (0.1e1 + del2 / D2 / 0.3e1) * pow(D1 * del1 * (D2 + del2 * A) * cosh(d1 / del1) + (pow(D1, 0.2e1) * del2 + D2 * pow(del1, 0.2e1) * A) * sinh(d1 / del1), -0.2e1) * (dD1d1 * del1 * (D2 + del2 * A) * cosh(d1 / del1) + D1 * ddel1d1 * (D2 + del2 * A) * cosh(d1 / del1) - D1 / del1 * (D2 + del2 * A) * sinh(d1 / del1) * d1 * ddel1d1 + (0.2e1 * D1 * del2 * dD1d1 + 0.2e1 * D2 * del1 * A * ddel1d1) * sinh(d1 / del1) - (pow(D1, 0.2e1) * del2 + D2 * pow(del1, 0.2e1) * A) * cosh(d1 / del1) * d1 * pow(del1, -0.2e1) * ddel1d1);


	*derivmuad = -((-(D2 * (((0.1e1 / 0.3e1 + A) * musr1 * pow(D2 + del2 / 0.3e1, 0.2e1) * D1 - del2 * D2 * ((0.1e1 / 0.3e1 + A) * del2 + 0.2e1 * D2) * musr2 / 0.9e1) * pow(del1, 0.2e1) + del2 * D2 * pow(D1, 0.2e1) * ((0.1e1 / 0.3e1 + A) * del2 + 0.2e1 * D2) * musr2) * ddel2d2 - del2 * (((0.1e1 / 0.3e1 + A) * musr1 * pow(D2 + del2 / 0.3e1, 0.2e1) * D1 + musr2 * del2 * (pow(del2, 0.2e1) * A - 0.3e1 * pow(D2, 0.2e1)) / 0.27e2) * pow(del1, 0.2e1) - musr2 * del2 * pow(D1, 0.2e1) * (pow(del2, 0.2e1) * A - 0.3e1 * pow(D2, 0.2e1)) / 0.3e1) * dD2d2) * D1 * del1 * cosh(d1 / del1) - sinh(d1 / del1) * (((musr1 * pow(D2 + del2 / 0.3e1, 0.2e1) * D1 - 0.2e1 / 0.3e1 * del2 * D2 * (D2 + del2 / 0.6e1) * musr2) * A * pow(del1, 0.4e1) + 0.6e1 * (musr1 * pow(D2 + del2 / 0.3e1, 0.2e1) * D1 / 0.2e1 + del2 * D2 * ((-0.1e1 / 0.18e2 + A / 0.6e1) * del2 + A * D2) * musr2) * pow(D1, 0.2e1) * pow(del1, 0.2e1) + 0.3e1 * musr2 * pow(del2, 0.2e1) * D2 * pow(D1, 0.4e1)) * D2 * ddel2d2 - del2 * dD2d2 * (A * (musr1 * pow(D2 + del2 / 0.3e1, 0.2e1) * D1 - musr2 * del2 * pow(D2, 0.2e1) / 0.3e1) * pow(del1, 0.4e1) + 0.3e1 * pow(D1, 0.2e1) * (musr1 * pow(D2 + del2 / 0.3e1, 0.2e1) * D1 + del2 * (pow(del2, 0.2e1) / 0.27e2 + pow(D2, 0.2e1) * A) * musr2) * pow(del1, 0.2e1) - pow(del2, 0.3e1) * musr2 * pow(D1, 0.4e1))) / 0.3e1) * exp(-d1 / D1 / 0.3e1) + (cosh(d1 / del1) - sinh(d1 / del1)) * (0.1e1 / 0.3e1 + A) * (cosh(d1 / del1) + sinh(d1 / del1)) * (ddel2d2 * D2 - del2 * dD2d2) * pow(D1, 0.2e1) * musr1 * pow(del1, 0.3e1) * pow(D2 + del2 / 0.3e1, 0.2e1)) * D1 * del1 * A * pow(D1 * del1 * (D2 + del2 * A) * cosh(d1 / del1) + (pow(D1, 0.2e1) * del2 + D2 * pow(del1, 0.2e1) * A) * sinh(d1 / del1), -0.2e1) / (D1 + del1 / 0.3e1) / (D1 - del1 / 0.3e1) * pow(D2 + del2 / 0.3e1, -0.2e1);

	
}


void ReflE2L2(float mua1, float mus1, float mua2, float mus2, float gcol, float *res, float *derivmuad, float *derivmuae){
	float d1 = de;
	float g1 = gcol;
	float g2 = gcol;
	float mutrr1 = mus1*(1-g1*g1) + mua1;
	float mutrr2 = mus2*(1-g2*g2) + mua2;
	float mutr1 = mus1*(1-g1) + mua1;
	float mutr2 = mus2*(1-g2) + mua2;
	float D1 = 1/(3*mutr1);
	float D2 = 1/(3*mutr2);
	float del1 = sqrt(D1/mua1);
	float del2 = sqrt(D2/mua2);
	float curr_muad = mua2;
	float curr_muae = mua1;
	
	/*float d2dD2 = -3.0f*D2*D2;
	float d2ddel2 = (d2dD2*curr_muad-D2)*1.0f/(curr_muad*curr_muad)*1.0f/(2.0f*del2);
	float d1dD1 = -3.0f*D1*D1;
	float d1ddel1 = (d1dD1*curr_muae-D1)*1.0f/(curr_muae*curr_muae)*1.0f/(2.0f*del1);
	float d1dmutr1 = mus1*(1-g1);
	float d2dmutr2 = mus2*(1-g2);
	float d1dmutrr1 = mus1*(1-g1*g1);
	float d2dmutrr2 = mus2*(1-g2*g2);
	float d1dK1 = -0.3e1 * del1 * mus1 * (double) (g1 - 1) * (del1 * (((double) (mutrr1 * D1 * g1) + 0.2e1 / 0.3e1 + 0.2e1 / 0.3e1 * (double) g1) * (double) mutrr1 * pow(del1, 0.2e1) + (double) (D1 * g1)) * (double) D1 * d1dmutrr1 + del1 * ((double) mutrr1 * del1 - 0.1e1) * ((double) mutrr1 * del1 + 0.1e1) * (double) (1 + g1) * d1dD1 / 0.3e1 + 0.2e1 * ((double) (mutrr1 * D1 * g1) + 0.1e1 / 0.3e1 + (double) g1 / 0.3e1) * d1ddel1 * (double) D1) * (double) (int) pow((double) D1, (double) (-2)) * pow(-0.1e1 + (double) (int) pow((double) mutrr1, (double) 2) * pow(del1, 0.2e1), -0.2e1);

	float d1dK2 = -mus2 * (double) (1 - g2 * g2) * (double) (1 / D2 + 3 * mutrr2 * g2 / (1 + g2)) * pow(del2, 0.2e1) / (0.1e1 - (double) (int) pow((double) mutrr2, (double) 2) * pow(del2, 0.2e1)) * d1dmutrr1 * d1 * exp(-mutrr1 * d1);
	float d2dK2 =  -0.3e1 * mus2 * ((mutrr2 * (mutrr2 * g2 * D2 + 0.2e1 / 0.3e1 + 0.2e1 / 0.3e1 * g2) * pow(del2, 0.2e1) + g2 * D2) * del2 * D2 * d2dmutrr2 + del2 * (mutrr2 * del2 - 0.1e1) * (mutrr2 * del2 + 0.1e1) * (0.1e1 + g2) * d2dD2 / 0.3e1 + 0.2e1 * (0.1e1 / 0.3e1 + g2 / 0.3e1 + mutrr2 * g2 * D2) * D2 * d2ddel2) * (g2 - 0.1e1) * del2 * exp(-mutrr1 * d1) * pow(D2, -0.2e1) * pow(-0.1e1 + pow(mutrr2, 0.2e1) * pow(del2, 0.2e1), -0.2e1);*/

	float K1 = mus1*(1-g1*g1)*(1/D1+3*mutrr1*g1/(1+g1))*del1*del1/(1-pow(mutrr1*del1,2));
	float K2 = mus2*(1-g2*g2)*(1/D2+3*mutrr2*g2/(1+g2))*del2*del2/(1-pow(mutrr2*del2,2))*exp(-mutrr1*d1);

	*res = -((double) A * ((double) ((D1 * K1 * mutrr1 * mutr1 - mus1 * (-1 + g1) * g1) * del1 + mutr1 * D1 * K1) * mutr2 * (double) (D1 * del2 - D2 * del1) * exp(-(double) (d1 / del1)) + mutr2 * (double) ((D1 * K1 * mutrr1 * mutr1 - mus1 * (-1 + g1) * g1) * del1 - mutr1 * D1 * K1) * (double) (D1 * del2 + D2 * del1) * exp((double) (d1 / del1)) - 0.2e1 * (double) del1 * (double) D1 * (mutr2 * (double) (D1 * mutr1 * del2 * K1 * mutrr1 - g1 * mus1 * (-1 + g1) * del2 - D2 * mutr1 * K1) * exp(-(double) (mutrr1 * d1)) - (double) mutr1 * (-(double) g2 * mus2 * (double) del2 * (double) (-1 + g2) * exp(-(double) (mutrr2 * d1)) + K2 * (double) D2 * mutr2 * (double) (-1 + mutrr2 * del2)))) / (double) mutr1 / mutr2 / ((double) (D1 * del2 - D2 * del1) * (double) (-D1 + A * del1) * exp(-(double) (d1 / del1)) + (double) (D1 * del2 + D2 * del1) * exp((double) (d1 / del1)) * (double) (D1 + A * del1)));



	float ddel2d2 = -sqrt(0.3e1) * (1 * mua2 + mutr2) * pow(0.1e1 / mutr2 / mua2, -0.1e1 / 0.2e1) * pow(mutr2, -0.2e1) * pow(mua2, -0.2e1) / 0.6e1;
	float ddel1d1 = -sqrt(0.3e1) * (1 * mua1 + mutr1) * pow(0.1e1 / mutr1 / mua1, -0.1e1 / 0.2e1) * pow(mutr1, -0.2e1) * pow(mua1, -0.2e1) / 0.6e1;
	float dD2d2 = -pow(mutr2, -0.2e1) * 1 / 0.3e1;
	float dD1d1 = -pow(mutr1, -0.2e1) * 1 / 0.3e1;

	float dK2d1 = -mus2 * (double) (1 - g2 * g2) * (double) (1 / D2 + 3 * mutrr2 * g2 / (1 + g2)) * pow(del2, 0.2e1) / (0.1e1 - (double) (int) pow((double) mutrr2, (double) 2) * pow(del2, 0.2e1)) * 1 * d1 * exp(-mutrr1 * d1);
	float dK2d2 = -0.3e1 * del2 * mus2 * (del2 * ((mutrr2 * g2 * D2 + 0.2e1 / 0.3e1 + 0.2e1 / 0.3e1 * g2) * mutrr2 * pow(del2, 0.2e1) + g2 * D2) * D2 * 1 + del2 * (-0.1e1 + mutrr2 * del2) * (mutrr2 * del2 + 0.1e1) * (0.1e1 + g2) * dD2d2 / 0.3e1 + 0.2e1 * ddel2d2 * D2 * (0.1e1 / 0.3e1 + g2 / 0.3e1 + mutrr2 * g2 * D2)) * exp(-mutrr1 * d1) * (-0.1e1 + g2) * pow(D2, -0.2e1) * pow(-0.1e1 + pow(mutrr2, 0.2e1) * pow(del2, 0.2e1), -0.2e1);
	float dK1d1 = -0.3e1 * del1 * (del1 * ((mutrr1 * D1 * g1 + 0.2e1 / 0.3e1 + 0.2e1 / 0.3e1 * g1) * mutrr1 * pow(del1, 0.2e1) + D1 * g1) * D1 * 1 + del1 * (mutrr1 * del1 - 0.1e1) * (mutrr1 * del1 + 0.1e1) * (0.1e1 + g1) * dD1d1 / 0.3e1 + 0.2e1 * (mutrr1 * D1 * g1 + 0.1e1 / 0.3e1 + g1 / 0.3e1) * ddel1d1 * D1) * (-0.1e1 + g1) * mus1 * pow(D1, -0.2e1) * pow(-0.1e1 + pow(mutrr1, 0.2e1) * pow(del1, 0.2e1), -0.2e1);

	*derivmuae = 0.2e1 * A * (-del1 * pow(D1 * del2 - D2 * del1, 0.2e1) * (-D1 * mutr1 * (D1 * K1 * mutrr1 * mutr1 + K1 * A * mutr1 - (double) (mus1 * (-1 + g1) * g1)) * ddel1d1 + del1 * (A * K1 * mutrr1 * mutr1 * del1 + K1 * A * mutr1 - (double) (mus1 * (-1 + g1) * g1)) * mutr1 * dD1d1 + (-D1 + A * del1) * (D1 * del1 * K1 * 1 * pow(mutr1, 0.2e1) + (double) g1 * del1 * (double) mus1 * (double) (-1 + g1) * 1 + D1 * dK1d1 * pow(mutr1, 0.2e1) * (mutrr1 * del1 + 0.1e1))) * mutr2 * pow(exp(-d1 / del1), 0.2e1) / 0.2e1 + (-0.2e1 * (((-D1 * D2 * del2 - d1 * pow(D2, 0.2e1)) * pow(del1, 0.2e1) + d1 * pow(del2, 0.2e1) * pow(D1, 0.2e1)) * D1 * mutr1 * (D1 * K1 * mutrr1 * mutr1 + K1 * A * mutr1 - (double) (mus1 * (-1 + g1) * g1)) * ddel1d1 + del1 * (pow(del1, 0.2e1) * (-A * pow(D2, 0.2e1) * K1 * mutrr1 * mutr1 * pow(del1, 0.2e1) + (K1 * mutrr1 * mutr1 * (A * del2 + 0.2e1 * D2) * D1 + 0.2e1 * (K1 * A * mutr1 - (double) (mus1 * (-1 + g1) * g1)) * D2) * D1 * del2) * mutr1 * dD1d1 + (D1 * del2 - D2 * del1) * (D1 * pow(del1, 0.2e1) * K1 * 1 * pow(mutr1, 0.2e1) * A + A * (double) g1 * pow(del1, 0.2e1) * (double) mus1 * (double) (-1 + g1) * 1 + D1 * dK1d1 * pow(mutr1, 0.2e1) * (D1 + mutrr1 * pow(del1, 0.2e1) * A)) * (D2 * del1 + D1 * del2)) / 0.2e1) * mutr2 * exp(d1 / del1) + (D1 * (-pow(del1, 0.3e1) * D2 * A - pow(del1, 0.2e1) * d1 * D2 * A + (D1 * del2 + d1 * (A * del2 + D2)) * D1 * del1 - pow(D1, 0.2e1) * del2 * d1) * (-K1 * mutrr1 * mutr1 * del2 * D1 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2 + K1 * mutr1 * D2) * mutr1 * ddel1d1 + pow(del1, 0.2e1) * ((D2 * (-0.2e1 * K1 * mutrr1 * mutr1 * del2 * D1 + K1 * mutr1 * D2 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2) * A * pow(del1, 0.2e1) + mutr1 * del2 * pow(D1, 0.2e1) * K1 * mutrr1 * (A * del2 + D2) * del1 - (K1 * mutr1 * D2 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2) * pow(D1, 0.2e1) * del2) * mutr1 * dD1d1 + (-D1 + A * del1) * (D1 * del2 - D2 * del1) * D1 * (mutr1 * (-mutr1 * del2 * K1 * (mutrr1 * d1 - 0.1e1) * D1 + (K1 * mutr1 * D2 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2) * d1) * 1 + (double) g1 * (double) mus1 * del2 * (double) (-1 + g1) * 1 + dK1d1 * pow(mutr1, 0.2e1) * (-D2 + D1 * del2 * mutrr1)))) * mutr2 * exp(-mutrr1 * d1) - (((-pow(del1, 0.3e1) * D2 * A - pow(del1, 0.2e1) * d1 * D2 * A + (D1 * del2 + d1 * (A * del2 + D2)) * D1 * del1 - pow(D1, 0.2e1) * del2 * d1) * D1 * ddel1d1 + pow(del1, 0.2e1) * dD1d1 * (D2 * A * pow(del1, 0.2e1) - del2 * pow(D1, 0.2e1))) * del2 * mus2 * (double) (-1 + g2) * (double) g2 * exp(-mutrr2 * d1) - (-(D1 * del2 - D2 * del1) * D1 * pow(del1, 0.2e1) * (-D1 + A * del1) * dK2d1 + ((-pow(del1, 0.3e1) * D2 * A - pow(del1, 0.2e1) * d1 * D2 * A + (D1 * del2 + d1 * (A * del2 + D2)) * D1 * del1 - pow(D1, 0.2e1) * del2 * d1) * D1 * ddel1d1 + pow(del1, 0.2e1) * dD1d1 * (D2 * A * pow(del1, 0.2e1) - del2 * pow(D1, 0.2e1))) * K2) * (-0.1e1 + mutrr2 * del2) * D2 * mutr2) * pow(mutr1, 0.2e1)) * exp(-d1 / del1) + exp(d1 / del1) * (-del1 * (D1 * mutr1 * (D1 * K1 * mutrr1 * mutr1 + K1 * A * mutr1 - (double) (mus1 * (-1 + g1) * g1)) * ddel1d1 + del1 * mutr1 * (A * K1 * mutrr1 * mutr1 * del1 - K1 * A * mutr1 + (double) (mus1 * (-1 + g1) * g1)) * dD1d1 + (D1 * del1 * K1 * 1 * pow(mutr1, 0.2e1) + (double) g1 * del1 * (double) mus1 * (double) (-1 + g1) * 1 + D1 * dK1d1 * pow(mutr1, 0.2e1) * (mutrr1 * del1 - 0.1e1)) * (D1 + A * del1)) * pow(D2 * del1 + D1 * del2, 0.2e1) * mutr2 * exp(d1 / del1) / 0.2e1 + (-(-pow(del1, 0.3e1) * D2 * A + pow(del1, 0.2e1) * d1 * D2 * A + (D1 * del2 + d1 * (A * del2 + D2)) * D1 * del1 + pow(D1, 0.2e1) * del2 * d1) * (-K1 * mutrr1 * mutr1 * del2 * D1 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2 + K1 * mutr1 * D2) * D1 * mutr1 * ddel1d1 + pow(del1, 0.2e1) * (-(D2 * (-0.2e1 * K1 * mutrr1 * mutr1 * del2 * D1 + K1 * mutr1 * D2 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2) * A * pow(del1, 0.2e1) - mutr1 * del2 * pow(D1, 0.2e1) * K1 * mutrr1 * (A * del2 + D2) * del1 - (K1 * mutr1 * D2 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2) * pow(D1, 0.2e1) * del2) * mutr1 * dD1d1 + (mutr1 * (-mutr1 * del2 * K1 * (mutrr1 * d1 - 0.1e1) * D1 + (K1 * mutr1 * D2 + (double) g1 * (double) mus1 * (double) (-1 + g1) * del2) * d1) * 1 + (double) g1 * (double) mus1 * del2 * (double) (-1 + g1) * 1 + dK1d1 * pow(mutr1, 0.2e1) * (-D2 + D1 * del2 * mutrr1)) * D1 * (D2 * del1 + D1 * del2) * (D1 + A * del1))) * mutr2 * exp(-mutrr1 * d1) + (((-pow(del1, 0.3e1) * D2 * A + pow(del1, 0.2e1) * d1 * D2 * A + (D1 * del2 + d1 * (A * del2 + D2)) * D1 * del1 + pow(D1, 0.2e1) * del2 * d1) * D1 * ddel1d1 + pow(del1, 0.2e1) * dD1d1 * (D2 * A * pow(del1, 0.2e1) - del2 * pow(D1, 0.2e1))) * del2 * mus2 * (double) (-1 + g2) * (double) g2 * exp(-mutrr2 * d1) - ((D2 * del1 + D1 * del2) * D1 * pow(del1, 0.2e1) * (D1 + A * del1) * dK2d1 + K2 * ((-pow(del1, 0.3e1) * D2 * A + pow(del1, 0.2e1) * d1 * D2 * A + (D1 * del2 + d1 * (A * del2 + D2)) * D1 * del1 + pow(D1, 0.2e1) * del2 * d1) * D1 * ddel1d1 + pow(del1, 0.2e1) * dD1d1 * (D2 * A * pow(del1, 0.2e1) - del2 * pow(D1, 0.2e1)))) * (-0.1e1 + mutrr2 * del2) * D2 * mutr2) * pow(mutr1, 0.2e1))) / del1 * pow((D1 * del2 - D2 * del1) * (-D1 + A * del1) * exp(-d1 / del1) + (D2 * del1 + D1 * del2) * exp(d1 / del1) * (D1 + A * del1), -0.2e1) * pow(mutr1, -0.2e1) / mutr2;

	*derivmuad = -0.2e1 * del1 * (double) D1 * (double) A * ((0.2e1 * del1 * (double) (K1 * (D1 * mutrr1 + A) * mutr1 - mus1 * (-1 + g1) * g1) * (-del2 * dD2d2 + ddel2d2 * D2) * (double) D1 * pow(mutr2, 0.2e1) * exp(d1 / del1) + (-(double) D1 + (double) A * del1) * ((mutr2 * del1 * ddel2d2 * D2 + (-mutr2 * del1 * dD2d2 + ((double) D1 * del2 - D2 * del1) * (1 + mutr2 * 1 * d1)) * del2) * mus2 * (double) mutr1 * (double) (-1 + g2) * (double) g2 * exp(-mutrr2 * d1) - ((-del2 * dD2d2 + ddel2d2 * D2) * ((double) (-D1 * K1 * mutrr1 * mutr1 + mus1 * (-1 + g1) * g1) * del1 + (double) (D1 * K1 * mutr1)) * exp(-(double) mutrr1 * d1) - ((-0.1e1 + mutrr2 * del2) * D2 * ((double) D1 * del2 - D2 * del1) * dK2d2 + K2 * (D2 * (-del1 * mutrr2 * D2 + (double) D1) * ddel2d2 + ((double) D1 * (-0.1e1 + mutrr2 * del2) * dD2d2 + 1 * D2 * ((double) D1 * del2 - D2 * del1)) * del2)) * (double) mutr1) * pow(mutr2, 0.2e1))) * exp(-d1 / del1) + ((-mutr2 * del1 * ddel2d2 * D2 + del2 * (mutr2 * del1 * dD2d2 + (D2 * del1 + (double) D1 * del2) * (1 + mutr2 * 1 * d1))) * mus2 * (double) mutr1 * (double) (-1 + g2) * (double) g2 * exp(-mutrr2 * d1) + ((-del2 * dD2d2 + ddel2d2 * D2) * ((double) (-D1 * K1 * mutrr1 * mutr1 + mus1 * (-1 + g1) * g1) * del1 - (double) (D1 * K1 * mutr1)) * exp(-(double) mutrr1 * d1) + ((-0.1e1 + mutrr2 * del2) * D2 * (D2 * del1 + (double) D1 * del2) * dK2d2 + (D2 * (del1 * mutrr2 * D2 + (double) D1) * ddel2d2 + ((double) D1 * (-0.1e1 + mutrr2 * del2) * dD2d2 + 1 * D2 * (D2 * del1 + (double) D1 * del2)) * del2) * K2) * (double) mutr1) * pow(mutr2, 0.2e1)) * ((double) D1 + (double) A * del1) * exp(d1 / del1)) / (double) mutr1 * pow(((double) D1 * del2 - D2 * del1) * (-(double) D1 + (double) A * del1) * exp(-d1 / del1) + (D2 * del1 + (double) D1 * del2) * exp(d1 / del1) * ((double) D1 + (double) A * del1), -0.2e1) * pow(mutr2, -0.2e1);
	



}

void ReflE2L3(float mua1, float mus1, float mua2, float mus2, float mua3, float mus3, float gcol, float *res, float d1, float d2){
	mutrr1 = mus1*(1-g1*g1)+mua1;
	mutrr2 = mus2*(1-g2*g2)+mua2;
	mutrr3 = mus3*(1-g3*g3)+mua3;

	mutr1 = mus1*(1-g1)+mua1;
	mutr2 = mus2*(1-g2)+mua2;
	mutr3 = mus3*(1-g3)+mua3;

	D1 = 1/(3*(mus1*(1-g1)+mua1));
	D2 = 1/(3*(mus2*(1-g2)+mua2));
	D3 = 1/(3*(mus3*(1-g3)+mua3));

	del1 = sqrt(D1/mua1); 
	del2 = sqrt(D2/mua2); 
	del3 = sqrt(D3/mua3);

	K1 = mus1*(1-g1*g1)*(1/D1+3*mutrr1*g1/(1+g1))*del1*del1/(1-mutrr1*mutrr1*del1*del1);
	K2 = mus2*(1-g2*g2)*(1/D2+3*mutrr2*g2/(1+g2))*del2*del2/(1-mutrr2*mutrr2*del2*del2)*exp(-mutrr1*d1);
	K3 = mus3*(1-g3*g3)*(1/D3+3*mutrr3*g3/(1+g3))*del3*del3/(1-mutrr3*mutrr3*del3*del3)*exp(-mutrr1*d1)*exp(-mutrr2*(d2-d1));
	
	gamma = -((exp(d2 / del2) * (D2 * del3 + D3 * del2) * (-D2 * del1 + D1 * del2) * ((D1 * mutr1 * K1 * mutrr1 - (mus1 * (-1 + g1) * g1)) * del1 + D1 * mutr1 * K1) * mutr2 * mutr3 * exp(-d1 / del1) - 0.4e1 * D1 * D2 * exp(d1 / del2) * del2 * mutr1 * del1 * mutr2 * mutr3 * K2 * (D2 * del3 * mutrr2 - D3) * exp(mutrr2 * (-d2 + d1)) + 0.4e1 * D2 * (g2 * del3 * mus2 * mutr3 * (-1 + g2) * exp(-mutrr2 * d2) + (-g3 * del3 * mus3 * (-1 + g3) * exp(-mutrr3 * d2) + D3 * K3 * mutr3 * (del3 * mutrr3 - 0.1e1)) * mutr2) * D1 * del1 * del2 * mutr1 * exp(d1 / del2) + exp(d2 / del2) * (D2 * del3 + D3 * del2) * ((D1 * del2 + D2 * del1) * ((D1 * mutr1 * K1 * mutrr1 - (mus1 * (-1 + g1) * g1)) * del1 - D1 * mutr1 * K1) * mutr2 * exp(d1 / del1) - 0.2e1 * ((D1 * mutr1 * del2 * K1 * mutrr1 - g1 * mus1 * (-1 + g1) * del2 - K1 * mutr1 * D2) * mutr2 * exp(-mutrr1 * d1) - (-g2 * mus2 * del2 * (-1 + g2) * exp(-mutrr2 * d1) + D2 * K2 * mutr2 * (mutrr2 * del2 - 0.1e1)) * mutr1) * D1 * del1) * mutr3) * exp(-d1 / del2) + exp(-d2 / del2) * ((D1 * del2 + D2 * del1) * ((D1 * mutr1 * K1 * mutrr1 - (mus1 * (-1 + g1) * g1)) * del1 + D1 * mutr1 * K1) * mutr2 * exp(-d1 / del1) + ((D1 * mutr1 * K1 * mutrr1 - (mus1 * (-1 + g1) * g1)) * del1 - D1 * mutr1 * K1) * (-D2 * del1 + D1 * del2) * mutr2 * exp(d1 / del1) - 0.2e1 * D1 * del1 * ((D1 * mutr1 * del2 * K1 * mutrr1 - g1 * mus1 * (-1 + g1) * del2 + K1 * mutr1 * D2) * mutr2 * exp(-mutrr1 * d1) - (-g2 * mus2 * del2 * (-1 + g2) * exp(-mutrr2 * d1) + D2 * K2 * mutr2 * (mutrr2 * del2 + 0.1e1)) * mutr1)) * exp(d1 / del2) * mutr3 * (D2 * del3 - D3 * del2)) * A / (((-D2 * del1 + D1 * del2) * (-D1 + A * del1) * exp(-d1 / del1) + (D1 * del2 + D2 * del1) * exp(d1 / del1) * (A * del1 + D1)) * exp(d2 / del2) * (D2 * del3 + D3 * del2) * exp(-d1 / del2) + exp(-d2 / del2) * ((D1 * del2 + D2 * del1) * (-D1 + A * del1) * exp(-d1 / del1) + (-D2 * del1 + D1 * del2) * exp(d1 / del1) * (A * del1 + D1)) * exp(d1 / del2) * (D2 * del3 - D3 * del2)) / mutr2 / mutr1 / mutr3;

	*res = gamma;
}

//ettlagsmodell
void ReflIsoL1(float mua, float mus, float gcol, float *res, float *deriv){
	float g = gcol;

	//reduced scattering coefficients
	float musr = mus*(1.0f-g);

	//move mua into shared memory
	float curr_mua = mua;
	
	//diffusion constant
	float D = 1.0f/(3.0f*(musr + curr_mua));

	//optical penetration depth
	float del = sqrt(D/(curr_mua));

	//from Svaasand 1995, modified
	//calculate the reflectance value
	float num = musr*A*del*del;
	float denom = (del/(3*D)+1)*(D+del*A);
	*res = num/denom;

	//calculate the derivative with respect to mua
	float dDdmua = -3.0f*D*D;
	float ddeldmua = (dDdmua*curr_mua-D)*1.0f/(curr_mua*curr_mua)*1.0f/(2.0f*del);
	float ddeldivDdmua = div13*(ddeldmua*D - del*dDdmua)*1.0f/(D*D);
	float dTdmua = musr*A*2*del*ddeldmua;
	float dNdmua = (dDdmua + ddeldmua*A)*(del/(3*D)+1)+(D+del*A)*ddeldivDdmua;
	*deriv = (dTdmua*denom - dNdmua*num)/(denom*denom);
	
//	*res = *res*(1-Rsp);
//	*deriv = *deriv*(1-Rsp);
}

//calculate the next muad based on the simulated reflectance, measured reflectance and derivative of the simulated reflectance using Newton-Rhapson
void nextMuad(float lineData, float refl, float deriv, float *muad, float safeinit){
	float currMuad = *muad;
	float currDeriv = deriv;
	float currLineData = lineData;
	float currRefl = refl;

	//newton's method
	currMuad = currMuad - (currRefl-currLineData)/currDeriv;
	if (currMuad <= 0.0f){
		currMuad = safeinit;
	}

	*muad = currMuad;
}


double muabEff(double r, double muab){
	return 1/(2*r)*(1-exp(-2*r*muab));
}

void calcSkinData(float lambda, float oxy1, float Bd1, float oxy2, float Bd2, float muam694, float bil2, float bil3, float car1, float car3, float met2, float met3, float wat1, float wat2, float wat3, float *mua1, float *mus1, float *mua2, float *mus2, float *mua3, float *mus3, float *gcol){
	double r1, r2, r3, r4;
	r1 = 10e-6;
	r2 = 25e-6;
	r3 = 50e-6;
	r4 = 100e-6;

	float muh_oxy = muh_oxy_calc(lambda);
	float muh_deoxy = muh_deoxy_calc(lambda);
	float melanin_base = pow((694/lambda),3.46);
	*gcol = 0.62 + lambda*29e-5;
	float musm = (1-1.745e-3*lambda + 9.843e-7*lambda*lambda)/(1-*gcol);
	float musr = pow(lambda, -4)/(1-*gcol); //NB FIXME!!!! Deler på 1-gcol!
	float musb_base = melanin(lambda);//pow((685/(lambda*1.0)), 0.37);
	
//absorption properties
	float H = 0.41;
	float H0 = 0.45;
	float Be = 0.002;
	float mua_other = 25; //FIXME

	//mua_other = (0.0244f + 8.53f*exp(-(lambda-154.0f)/66.2f))*1000;//Huang background absorption

	float muab_blood1 = (muh_oxy*oxy1 + muh_deoxy*(1-oxy1))*H/H0;
	//muab_blood1 = 0.7*muabEff(r1, muab_blood1) + 0.24*muabEff(r2, muab_blood1) + 0.04*muabEff(r3, muab_blood1) + 0.02*muabEff(r4, muab_blood1); //shielding
	float muab_blood2 = (muh_oxy*oxy2 + muh_deoxy*(1-oxy2))*H/H0;
	//muab_blood2 = 0.7*muabEff(r1, muab_blood2) + 0.24*muabEff(r2, muab_blood2) + 0.04*muabEff(r3, muab_blood2) + 0.02*muabEff(r4, muab_blood2); //shielding
	float mua_melanin = muam694*melanin_base;
	*mua1 = mua_melanin + muab_blood1*Be + mua_other*(1-Be);
	*mua2 = muab_blood1*Bd1 + mua_other*(1-Bd1);
	*mua3 = muab_blood2*Bd2 + mua_other*(1-Bd2);

	float bet_chrom = betacarot(lambda);
	float bil_chrom = bilirub(lambda);
	float met_chrom = methb(lambda);
	float wat_chrom = water(lambda);

	*mua1 += bet_chrom*car1 + wat_chrom*wat1; 
	*mua2 += met_chrom*met2 + bil_chrom*bil2 + wat_chrom*wat2;
	*mua3 += bet_chrom*car3 + met_chrom*met3 + bil_chrom*bil3 + wat_chrom*wat3;

	//scattering properties
	float c_ray = 1.05e12;
	float c_mie = 105;
	float must = musm*c_mie*100 + musr*c_ray*100;
	//float f = 0.34586;
	//float b = 0.91;
	//float musrjaq = 5587.4;
	//must = musrjaq*(f*pow(lambda/500,-4) + (1-f)*pow(lambda/500, -b))/(1-*gcol);
	float musb685 = 55.09e-12;
	float ve = 1.25e-16;
	float musb = musb685*H*(1-H)*(1.4-H)*1.0f/ve*musb_base;
	*mus1 = must*(1-Be)+musb*Be;
	*mus2 = must*(1-Bd2)+musb*Bd1;//*(1-0.9949)/(1-*gcol);
	*mus3 = must*(1-Bd2)+musb*Bd2;//*(1-0.9949)/(1-*gcol);
}

void calcSkinData(float lambda, float oxy1, float Bd1, float oxy2, float Bd2, float muam694, float bil, float car, float met, float wat, float *mua1, float *mus1, float *mua2, float *mus2, float *mua3, float *mus3, float *gcol){
	calcSkinData(lambda, oxy1, Bd1, oxy2, Bd2, muam694, 0.0, bil, 0.0, car, 0.0, met, 0.0, 0.0, wat, mua1, mus1, mua2, mus2, mua3, mus3, gcol);
}
