#include <gsl/gsl_multifit.h>
#include<gsl/gsl_blas.h>
#include <iostream>
using namespace std;

bool nnls_hanson(gsl_matrix *chromophores, gsl_vector *absspec, int param, int obs, gsl_vector *x, double *std){
	gsl_vector *w = gsl_vector_alloc(param);

	bool *P = new bool[param]; //active set
	for (int i=0; i < param; i++){
		P[i] = 0;
		gsl_vector_set(x, i, 0);
		gsl_vector_set(w, i, 0);
	}

	bool failed = false;

	//find error vector w
	gsl_matrix *A_transp = gsl_matrix_alloc(param, obs);
	gsl_matrix_transpose_memcpy(A_transp, chromophores);
	double *temparr = new double[obs];
	for (int i=0; i < obs; i++){	
		double tempval = 0;
		for (int j=0; j < param; j++){
			tempval += gsl_matrix_get(chromophores, i, j)*gsl_vector_get(x,j);
		}
		temparr[i] = gsl_vector_get(absspec, i) - tempval;
		for (int j=0; j < param; j++){
			gsl_vector_set(w, j, gsl_vector_get(w, j) + gsl_matrix_get(A_transp, j, i)*temparr[i]);
		}
	}

	bool RIsEmpty = false;
	bool allWActiveNeg = false; //whether all w in the active set are less or equal to zero or not

	int maxIter = 50;
	int iter = 0;
	while (!RIsEmpty && (!allWActiveNeg) && (iter < maxIter)){
		//find maximum error w
		int indMaxW = 0; //index of maximum w
		float maxW = -1000000000;
		for (int i=0; i < param; i++){
			if (!P[i]){
				if (gsl_vector_get(w, i) > maxW){
					indMaxW = i;
					maxW = gsl_vector_get(w,i);
				}
			}
		}

		//include index indMaxW in P, remove from R
		P[indMaxW] = 1;
		
		//find number of passive params
		int passive_params = 0;
		for (int i=0; i < param; i++){
			if (P[i]){
				passive_params++;
			}
		}
		//create matrix for the passive components, transfer columns from the component matrix to passive matrix
		gsl_matrix *A_passive = gsl_matrix_alloc(obs, passive_params);
		int j = 0; //current index in A_passive
		for (int i=0; i < param; i++){
			if (P[i]){
				gsl_vector *v = gsl_vector_alloc(obs);
				gsl_matrix_get_col(v, chromophores, i);
				gsl_matrix_set_col(A_passive, j, v);
				gsl_vector_free(v);
				j++;
			}
		}
		//solve linear least squares problem for the passive parameters
		gsl_multifit_linear_workspace *workspace = gsl_multifit_linear_alloc(obs, passive_params);
		gsl_matrix *cov = gsl_matrix_alloc(passive_params, passive_params);
		double chisq = 0;
		gsl_vector *c_passive = gsl_vector_alloc(passive_params);
		gsl_multifit_linear(A_passive, absspec, c_passive, cov, &chisq, workspace);

		//continue until P only actually has passive parameters
		float min_c_passive = gsl_vector_min(c_passive);
		int innerIter = 0;
		int lastalfaInd = 0;
		while ((min_c_passive <= 0) && (innerIter < maxIter)){
			float alfa = 100000000;
			int j=0;

			//find alfa
			for (int i=0; i < param; i++){
				if (P[i]){
					float newVal = gsl_vector_get(x,i)/(gsl_vector_get(x,i)-gsl_vector_get(c_passive,j));
					if ((newVal < alfa) && (gsl_vector_get(c_passive, j) <= 0)){
						alfa = newVal;
						lastalfaInd = i;
					}
					j++;
				}
			}

			//iterate x
			j=0;
			for (int i=0; i < param; i++){
				if (P[i]){
					gsl_vector_set(x,i,gsl_vector_get(x, i) + alfa*(gsl_vector_get(c_passive, j) - gsl_vector_get(x, i)));
					j++;
				} else {
					gsl_vector_set(x,i,(1-alfa)*gsl_vector_get(x, i));
				}
			}

			//find the xs which are still positive, move 'em to the passive set
			passive_params = 0;
			for (int i=0; i < param; i++){
				P[i] = gsl_vector_get(x, i) > 0;
				//since there will probably be numerical inaccuracies, do at least ensure that the index corresponding to the alpha no longer is a part of the passive set (since x at this position WILL be zero)
				if (i == lastalfaInd){
					P[i] = 0;
				}
				passive_params += P[i];
			}
			if (passive_params == 0){
				//something went wrong
				for (int i=0; i < param; i++){
					gsl_vector_set(x, i, 1.0f);
				}
				failed = true;
				break;
			}

			/*
			cout << "p: ";
			for (int i=0; i < param; i++){
				cout << P[i] << " ";
			}
			cout << endl;*/
			
			//freeing memory
			gsl_multifit_linear_free(workspace);
			gsl_matrix_free(cov);
			gsl_matrix_free(A_passive);
			gsl_vector_free(c_passive);

			//create new matrices for new fittin'
			A_passive = gsl_matrix_alloc(obs, passive_params);
			j = 0; //current index in A_passive
			for (int i=0; i < param; i++){
				if (P[i]){
					gsl_vector *v = gsl_vector_alloc(obs);
					gsl_matrix_get_col(v, chromophores, i);
					gsl_matrix_set_col(A_passive, j, v);
					gsl_vector_free(v);
					j++;
				}
			}
			workspace = gsl_multifit_linear_alloc(obs, passive_params);
			cov = gsl_matrix_alloc(passive_params, passive_params);
			c_passive = gsl_vector_alloc(passive_params);
			gsl_multifit_linear(A_passive, absspec, c_passive, cov, &chisq, workspace);
			min_c_passive = gsl_vector_min(c_passive);

			innerIter++;
			/*
			cout << "last alfa ind: " << lastalfaInd << endl;
			cout << "x: ";
			for (int i=0; i < param; i++){
				cout << gsl_vector_get(x,i) << " ";
			}
			cout << endl;
			cout << "c_passive: ";
			for (int i=0; i < passive_params; i++){
				cout << gsl_vector_get(c_passive,i) << " ";
			}
			cout << endl;*/
		}
		if (innerIter == maxIter){
			//cerr << "WARNING! Jumped out of inner iteration." << endl;
			failed = true;
		}

		//transfer c_passive and c_active (=== 0) to x
		j = 0;
		for (int i=0; i < param; i++){
			if (P[i]){
				gsl_vector_set(x, i, gsl_vector_get(c_passive, j));
				j++;
			} else {
				gsl_vector_set(x, i, 0);
			}
			gsl_vector_set(w, i, 0);
		}

		//calculate w 
		for (int i=0; i < obs; i++){	
			double tempval = 0;
			for (int j=0; j < param; j++){
				tempval += gsl_matrix_get(chromophores, i, j)*gsl_vector_get(x,j);
			}
			temparr[i] = gsl_vector_get(absspec, i) - tempval;
			for (int j=0; j < param; j++){
				gsl_vector_set(w, j, gsl_vector_get(w, j) + gsl_matrix_get(A_transp, j, i)*temparr[i]);
			}
		}

		//find negative ws among the active parameters and number of active parameters
		int active_param = 0;
		int neg_param = 0;
		j=0; 
		for (int i=0; i < param; i++){
			if (!P[i]){
				if (gsl_vector_get(w, i) <= 0){
					neg_param++;
				}
				active_param++;
				std[i] = 0.0;
			} else {
				std[i] = fabs(gsl_vector_get(w, i));
				j++;
			}
		}
		allWActiveNeg = neg_param == active_param;

		RIsEmpty = active_param == 0;
		iter++;

		//free memory
		gsl_multifit_linear_free(workspace);
		gsl_matrix_free(cov);
		gsl_vector_free(c_passive);
		gsl_matrix_free(A_passive);
	}
	if (iter == maxIter){
		//cerr << "WARNING! Jumped out of outer iteration." << endl;
		failed = true;
	}
	gsl_vector_free(w);
	gsl_matrix_free(A_transp);
	delete [] P;
	delete [] temparr;
	for (int i=0; i < param; i++){
		if ((gsl_vector_get(x,i) < 0) || isnan(gsl_vector_get(x,i))){
			//cerr << "Warning! Negative or nan values: " << i << " " << gsl_vector_get(x,i) << endl;
			//cerr << param << " " << obs << endl;
		}
	}
	
	return !failed;
}






