//calibration
void CalibrationStage::calibrate(float *data, float *calibrated){
	//calibrate against calibration array
	for (int band=0; band < _bands; band++){
		int start, end;
		start = 0;
		end = _imageInputSamples;
		for (int pix=start; pix < end; pix++){
			float val = 0;
			val = 1.0f*(float)data[band*_imageInputSamples + pix];
			val /= _calarray[band*_samples + pix];
			calibrated[band*_samples + pix] = val;
		}
	}	
	
	//threshold the calibrated array, keep only skin
	bool *skinPresence = skinThresh(calibrated);
	for (int band=0; band < _bands; band++){
		for (int pix = 0; pix < _imageInputSamples; pix++){
			if (!skinPresence[pix]){
				calibrated[band*_samples + pix] = 0.0f;
			} 
		}
	}
	delete [] skinPresence;
}

void CalibrationStage::execute(){
	//omitted
}

//threshold skin
bool* CalibrationStage::skinThresh(float *inputImage){
	bool *retarr = new bool[_samples];
	//take spectrum from the middle of the hyperspectral image, subtract its mean
	float *arr = new float[_bands];
	int sampleSize = 50;
	float mean = 0;
	for (int i=0; i < _bands; i++){
		arr[i] = 0.0f;
		int num = 0;
		for (int j=_samples/2-sampleSize; j < _samples/2+sampleSize; j++){
			float inp = inputImage[i*_samples + j];
			if ((inp >= 0.0f) && (inp <= 1.0f)){
				arr[i] += inp;
				num++;
			}
		}
		if (num == 0){
			num = 1;
		}
		arr[i] /= num;
		mean += arr[i];
	}
	mean /= _bands;
	float arrnorm = 0;
	for (int i=0; i < _bands; i++){
		arr[i] -= mean;
		arrnorm += arr[i]*arr[i];

	}

	float thresh = 0.60; //arbitrary threshold value, could be more dynamic

	//multiply spectrum by the spectrum in each pixel across the entire field of view
	for (int i=0; i < _samples; i++){
		float temp = 0;
		float *temparr = new float[_bands];

		
		//subtract spectrum by its mean. also find norm of vector
		float mean = 0;
		for (int j=0; j < _bands; j++){
			mean += inputImage[j*_samples + i];
		}
		mean /= _bands;
		
		float tempnorm = 0;
		for (int j=0; j < _bands; j++){
			temparr[j] = inputImage[j*_samples + i] - mean;
			tempnorm += temparr[j]*temparr[j];
		}

		//multiply resulting spectrum by the spectrum found earlier from the middle of the image
		for (int j=0; j < _bands; j++){
			temp += temparr[j]*arr[j];
		}
		temp /= sqrt(tempnorm)*sqrt(arrnorm);
		delete [] temparr;

		retarr[i] = temp > thresh;
	}
	delete [] arr;
	return retarr;
}

//for detection of a square slab with known placement
float* CalibrationStage::easyDetect(float *inputImage, int height){
	const int START_LINE = 0;
	const int END_LINE = 420;
	float *outputCalarr = new float[_samples*_bands];
	float factor = 0.99;
	for (int i=0; i < _samples; i++){
		for (int j=0; j < _bands; j++){
			outputCalarr[j*_samples + i] = 0.0f;
			int num = 0;
			for (int k=START_LINE; k < END_LINE; k++){
				outputCalarr[j*_samples + i] += inputImage[k*_samples*_bands + _samples*j + i];
				if (j == 100){
					file << inputImage[k*_samples*_bands + _samples*j + i] << " ";
				}
				num++;
			}
			outputCalarr[j*_samples + i] /= num*1.0f*factor;
		}
	}
	return outputCalarr;
}


#define calcmax(arg) if (inputImage[i*width*_bands + arg*width + j] > max##arg){\
			max##arg = inputImage[i*width*_bands + arg*width + j];\
			}

//for detection of paper stripe and circular reflectance standard with unknown placements
//FIXME NB: the bands are fixed! This will be the destroyer of everything you hold dear in the future. what is wrong with you 
float* CalibrationStage::detectan(float *inputImage, int height, int width){
	//readyment
	unsigned char *band0im = new unsigned char[height*width];
	unsigned char *band30im = new unsigned char[height*width];
	unsigned char *band50im = new unsigned char[height*width];
	unsigned char *band60im = new unsigned char[height*width];
	float max0 = 0;
	float max30 = 0;
	float max50 = 0;
	float max60 = 0;
	for (int i=0; i < height; i++){
		for (int j=0; j < width; j++){
			calcmax(0)
			calcmax(30)
			calcmax(50)
			calcmax(60)
		}
	}
	for (int i=0; i < height; i++){
		for (int j=0; j < width; j++){
			band0im[i*width + j] = (unsigned char)(inputImage[i*width*_bands + 0*width + j]/max0*255);
			band30im[i*width + j] = (unsigned char)(inputImage[i*width*_bands + 30*width + j]/max30*255);
			band50im[i*width + j] = (unsigned char)(inputImage[i*width*_bands + 50*width + j]/max50*255);
			band60im[i*width + j] = (unsigned char)(inputImage[i*width*_bands + 60*width + j]/max60*255);
		}
	}
	Mat band0(height, width, CV_8UC1, band0im);
	Mat band30(height, width, CV_8UC1, band30im);
	Mat band50(height, width, CV_8UC1, band50im);
	Mat band60(height, width, CV_8UC1, band60im);

	Mat band0sm, band30sm, band50sm, band60sm;
	GaussianBlur(band0, band0sm, Size(9, 9), 2, 2);
	GaussianBlur(band30, band30sm, Size(9, 9), 2, 2);
	GaussianBlur(band50, band50sm, Size(9, 9), 2, 2);
	GaussianBlur(band60, band60sm, Size(9, 9), 2, 2);

	//throw your skin away!
	int thresh1 = 0.23*255; //FIXME: choose something based on a minimum (opencv has a convenience function for this)
	cerr << "skinthresh" << endl;
	Mat skinthresh = segment(band0sm, Mat(), thresh1, false);

	int erosion_size = 10;
	int erosion_type = MORPH_RECT; //MORPH_CROSS, MORPH_ELLIPSE

	Mat element = getStructuringElement(erosion_type, Size(2*erosion_size + 1, 2*erosion_size+1), Point(erosion_size, erosion_size));
	int thresh12 = 0.37*255;
	Mat restthresh = segment(band0sm, Mat(), thresh12, false);
	Mat temp; 
	erode(restthresh, temp, element);
	restthresh = temp;

	//cut out pink junk
	int thresh2 = 0.47*255;
	cerr << "junkthresh" << endl;
	Mat junkthresh = segment(band30sm, skinthresh, thresh2, true);
	Mat junkthresh2 = segment(band30sm, restthresh, thresh2, true);

	//throw out pillow
	int thresh3 = 0.6*255;
	cerr << "pillowthresh" << endl;
	Mat pillowthresh = segment(band50sm, junkthresh, thresh3, true);

	//pillowthresh can now be used to calculate the calibration array from a calibration slab with a high multipliaction factor
	float *calslab = new float[_samples];
	for (int i=0; i < _samples; i++){
		calslab[i] = 0;
	}

	//actually, just use junkthresh instead
	unsigned char *presence = (unsigned char*)junkthresh2.data;
	showIm(junkthresh2, wndname1);
	float sheetfact = 0.99;
	int *number = new int[_samples]; //the number of samples used in each calibration mean
	int kalband = 50;

	float *test = new float[_bands*_samples];
	for (int i=0; i < _bands*_samples; i++){
		test[i] = 0.0;
	}
	ofstream fileslab;
	fileslab.open("output/slab.dat");
	for (int j=0; j < width; j++){
		number[j] = 0;
		for (int i=height/3; i < height/2; i++){
			fileslab << (int)presence[i*width + j] << " ";
			if ((int)presence[i*width + j]){
				calslab[j] += inputImage[i*_samples*_bands + kalband*_samples + j]/sheetfact;
				number[j]++;
			}
		}
		fileslab << endl;
		calslab[j] /= number[j];
	}

	//run moving average on resulting calslab
	int movSize = 50;
	int maxplacement = 0;
	float max = 0;
	for (int j=0; j < _samples; j++){
		float temp = 0;
		int num = 0;
		for (int i=-movSize; i <= movSize; i++){
			if (((i+j) >= 0) && ((i+j) < _samples)){
				temp += calslab[j + i];
				num++;
			}
		}
		temp /= num;
		calslab[j] = temp;
		if (temp > max){
			max = temp;
		}
	}
	for (int j=0; j < _samples; j++){
		calslab[j] /= max;
	}

	delete [] number;

	//enhance the circle presence
	int thresh4 = 0.5*255;
	Mat negJunkthresh;
	Mat skinNegJunkThresh;
	bitwise_not(junkthresh, negJunkthresh);
	Mat goodcirc = segment(band60sm, skinthresh, thresh4, true);
	Mat junkseg;
	bitwise_and(goodcirc, negJunkthresh, junkseg);
	floodFill(junkseg, Point(0,0), 255);
	Mat disk;
	bitwise_not(junkseg, disk);
	Mat eroded;
	erode(disk, eroded, element);
	Mat dilated;
	dilate(eroded, dilated, element);
	disk = dilated;
	
	//disk is now used to find the calibration array of a low multiplication factor
	float calfact = _calfactor;
	float *calarr = new float[_bands];
	presence = (unsigned char*)disk.data;
	ofstream filecal;
	filecal.open("output/cal.dat");
	int num = 0;
	for (int i=0; i < _bands; i++){
		calarr[i] = 0;
	}
	for (int j=0; j < width; j++){
		for (int i=0; i < height; i++){
			filecal << (int)presence[i*width + j] << " ";
			if (presence[i*width + j]){
				for (int k=0; k < _bands; k++){
					calarr[k] += inputImage[i*_samples*_bands + k*_samples + j]/calfact/calslab[j];
				}
				num++;
			}
		}
		filecal << endl;
	}

	for (int k=0; k < _bands; k++){
		calarr[k] /= num;
	}
	
	float *outputCalarr = new float[_samples*_bands];
	for (int i=0; i < _samples; i++){
		for (int j=0; j < _bands; j++){
			outputCalarr[j*_samples + i] = calslab[i]*calarr[j];
		}
	}

	
	delete [] calarr;
	delete [] calslab;

	delete [] band0im;
	delete [] band30im;
	delete [] band50im;
	delete [] band60im;

	return outputCalarr;
	_calfactor = 1.0;
}


Mat CalibrationStage::segment(Mat inputImage, Mat inputBinary, int tresh, bool shouldMult){
	//convert to image matrix
	Mat thresholded;
	Mat image;
	if (shouldMult){
		inputImage.copyTo(image, inputBinary);
	} else {
		image = inputImage;
	}
	
	//threshold the image based on some value
	int const max_BINARY_value = 255;
	int threshold_type = 0; //result is binary
	threshold(image, thresholded, tresh, max_BINARY_value, threshold_type);
	return thresholded;
}
