
//remove muam694-component from input mua
__global__ void removeMuam(float *mua, float *endvalues, float *AT, int startblockInd, int numBands, int endmembers, int muamInd, size_t pitch){
	int ind = blockIdx.x*pitch + threadIdx.x;
	int pixind = ind + gridDim.x*(blockIdx.y+startblockInd)*pitch;
	float muaval = 0;
	for (int i=0; i < endmembers; i++){
		__shared__ float temp;
		if (threadIdx.x == 0){
			temp = AT[i*numBands + blockIdx.y];
		}
		__syncthreads();
		muaval += temp*endvalues[ind+i*gridDim.x*pitch]*(i != muamInd);
	}
	mua[pixind] = muaval;
}
//perform moving average of size samples on the input array
template<int BLOCK_DIM>
__global__ void movingAverage(float *arr, float *outarr, int samplesdiv2, size_t pitch){
	__shared__ float shArrvals[3*BLOCK_DIM];
	shArrvals[threadIdx.x + blockDim.x] = arr[blockIdx.x*pitch + threadIdx.x];
	shArrvals[threadIdx.x + 2*blockDim.x] = 0;
	shArrvals[threadIdx.x] = 0;

	//read in the next block
	if (blockIdx.x < gridDim.x-1){
		shArrvals[threadIdx.x + 2*blockDim.x] = arr[(blockIdx.x+1)*pitch + threadIdx.x];
	}

	//read in the previous block
	if (blockIdx.x > 0){
		shArrvals[threadIdx.x] = arr[(blockIdx.x-1)*pitch + threadIdx.x];
	}
	__syncthreads();


	float newval = 0;

	//appropriate start and end indices in shared array
	int startind = (blockDim.x + threadIdx.x) - samplesdiv2;
	int endind = (blockDim.x + threadIdx.x) + samplesdiv2;

	for (int i=startind; i <= endind; i++){
		float temp = shArrvals[i];
		newval += !isnan(temp)*temp;
	}
	outarr[blockIdx.x*pitch + threadIdx.x] = fdividef(newval,2*samplesdiv2+1);
	
}


//deinitializing muam
__global__ void muamDeinit(float *muam, size_t pitch){
	int ind = blockIdx.x*pitch+ threadIdx.x;
	muam[ind] = 200.0f;
}

__global__ void skindataDeinit(float *muam, float *bvf, float *oxy, float *melanin_type, size_t pitch){
	int ind = blockIdx.x*pitch+ threadIdx.x;
	muam[ind] = 100.0f;
	bvf[ind] = 0.01;
	oxy[ind] = 0.8;
	melanin_type[ind] = SVAASAND_MELANIN_GPU;

}

//check scaling of muad at wavelengths 1 and 2 and move the melanin type up one step. PHEOMELANIN_GPU -> SVAASAND_MELANIN_GPU -> EUMELANIN_GPU -> ??? wrong scattering
__global__ void CheckReflectanceScaling(float *muad, float *melanin_type, int wlenind1, int wlenind2, size_t pitch, float scaling_thresh){
	int ind1 = (gridDim.x*wlenind1 + blockIdx.x)*pitch + threadIdx.x;
	int ind2 = (gridDim.x*wlenind2 + blockIdx.x)*pitch + threadIdx.x;
	float scaling = fdividef(muad[ind1],muad[ind2]);
	melanin_type[blockIdx.x*pitch + threadIdx.x] += scaling < scaling_thresh;
}

__global__ void SetMelaninType(float *melanin_type, int inputType, size_t pitch){
	melanin_type[blockIdx.x*pitch + threadIdx.x] = inputType;	
}

//penetration depth calculations
__device__ float calcPenDepth(float muae, float muad, float musd, float gcol){
	return sqrt(fdividef(1.0f, 3*(musd*(1-gcol) + muad)*(muad)));
}

__global__ void PenDepths(float *muae, float *muad, float *muse, float *musd, float *gcol, int startwlenind, int endwlenind, size_t pitch, float *pendepths, float *pendepthsstd){
	int pixind = pitch*blockIdx.x + threadIdx.x;
	float pendepth = 0;
	int ant = endwlenind - startwlenind;
	float penmax = 0;
	float penmin = 1000;
	for (int i=startwlenind; i < endwlenind; i++){
		int ind = pixind + gridDim.x*pitch*i;
		float temp = calcPenDepth(muae[ind], muad[ind], musd[ind], gcol[ind]);
		pendepth += temp;
		if (temp > penmax){
			penmax = temp;
		} 
		if (temp < penmin){
			penmin = temp;
		}
	}
	float diff = penmax - penmin;
	pendepths[pixind] = pendepth/(ant*1.0f);
	pendepthsstd[pixind] = diff;
}

__global__ void GPUlseunoptim(float *lsemat, float *input, float *res, float startWlen, float stepWlen, int antWlens, int *uppers, int *downers, float *upperwlens, float *downerwlens, size_t pitch, size_t intPitch){
	int i=0;
	int lseInd;
	//int inputInd;
	float restemp = 0;

	for (i = 0; i < antWlens; i++){
		//interpolate
		int interpolInd = i*intPitch + threadIdx.x;
		float w = startWlen + i*stepWlen;
		float wupp = downerwlens[interpolInd];
		float wdown = upperwlens[interpolInd];
		int uppind = (gridDim.x*downers[interpolInd] + blockIdx.x)*pitch + threadIdx.x; 
		int downind = (gridDim.x*uppers[interpolInd] + blockIdx.x)*pitch + threadIdx.x;
		float inpupp = input[uppind];
		float inpdow = input[downind];
		float val = inpupp + fdividef(w-wdown, wupp-wdown)*(inpupp-inpdow);
		
		//access lse matrix
		lseInd = blockIdx.y*antWlens*pitch + i*pitch + threadIdx.x; //parameter + wavelength + piksel
		float temp = lsemat[lseInd];
		restemp += temp*val;
	}
	int paramInd = blockIdx.y*pitch*gridDim.x + blockIdx.x*pitch + threadIdx.x; 
	res[paramInd] = restemp;
}

//proever aa lese ut fra reflektansmatrisa saa faa ganger som mulig. Usikkert om forloekken gjoer at det blir like tregt uansett
__global__ void GPUlseOptim(float *lsemat, float *input, float *res, int startind, int endind, int *numWlens, float *wlens, float *startwlens, int antWlens, size_t pitch, size_t intPitch){
	int i,j;
	int k=0;
	float restemp = 0;
	for (i=startind; i < endind; i++){
		int wlenind = i*pitch + threadIdx.x;
		int wlenindNext = (i+1)*pitch + threadIdx.x;
		int indind = i*intPitch + threadIdx.x;
		
		int wlenind2 = (gridDim.x*i + blockIdx.x)*pitch + threadIdx.x;
		int wlenindNext2 = (gridDim.x*(i+1) + blockIdx.x)*pitch + threadIdx.x;
		
		float startwlen = startwlens[wlenind];
		int numWlen = numWlens[indind];
		float wdown = wlens[wlenind];
		float wupp = wlens[wlenindNext];
		float rdown = input[wlenind2];
		float rupp = input[wlenindNext2];

		for (j=0; j < numWlen; j++){
			float val = rupp + fdividef(startwlen + j - wdown, wupp-wdown)*(rupp - rdown);
			int lseInd = blockIdx.y*antWlens*pitch + k*pitch + threadIdx.x;
			restemp += lsemat[lseInd]*val;
			k++;
		}
	}
	int paramInd = blockIdx.y*pitch*gridDim.x + blockIdx.x*pitch + threadIdx.x; 
	res[paramInd] = k;
}

__global__ void calcOxyBvf(float *inputRes, float *oxyOut, float *bvfOut, size_t pitch){
	int ind = blockIdx.x*pitch + threadIdx.x;
	float foxy = inputRes[ind];
	float fdeoxy = inputRes[ind + gridDim.x*pitch];
	float bvf = foxy + fdeoxy;
	float oxy = fdividef(foxy, bvf);
	bvf = (bvf >= 0)*(bvf <= 1)*bvf;
	oxy = (oxy >= 0)*(oxy <= 1)*oxy;
	oxyOut[ind] = oxy;
	bvfOut[ind] = bvf;
}

#define MAX_ITER 1000
//Tatt rett fra Gonzalez2013 for aa gjoere det enkelt saa jeg vet hva jeg har aa sammenligne meg med, ingen vits aa bruke mer tid paa ISRA
template<int MAX_NUM_BANDS>
__global__ void ISRA(float *mua, float *endout, float *chromarr, size_t pitch, int startwlenInd, int numEndmembers, int numBands){
	float s_end[MAX_NUM_BANDS]; float s_endt[MAX_NUM_BANDS]; 
	float numerator; float denominator; float dot = 0.0;
	int pixind = blockIdx.x*pitch + threadIdx.x;
	
	for (int e=0; e < numEndmembers; e++){
		int endind = pixind + gridDim.x*pitch*e;
		endout[endind] = 0;
	}

	for (int it=0; it <= MAX_ITER; it++){
		numerator = 0; denominator = 0;
		for (int e=0; e < numEndmembers; e++){
			int endind = pixind + gridDim.x*pitch*e;
			float currEnd = endout[endind];

			if (threadIdx.x == 0){
				for (int i=0; i < numBands; i++){
					s_end[i] = chromarr[e*numBands+i];
				}
			}
			__syncthreads();

			for (int k=0; k < numBands; k++){
				int bandind = pixind + gridDim.x*pitch*(startwlenInd + k);
				numerator = numerator + s_end[k]*mua[bandind];

				if (threadIdx.x == 0){
					for (int i=0; i < numEndmembers; i++){
						s_endt[i] = chromarr[k+i*numBands];
					}
				}
				__syncthreads();

				for (int s=0; s < numBands; s++){
					dot += s_endt[s]*currEnd;
				}
				denominator += dot*s_end[k];
				dot = 0;
			}
			endout[endind] = currEnd*numerator/denominator;
			numerator = 0;
			denominator = 0;
		}
	}
}

#define USE_SHMEM
#define USE_LOCMEM
#define NUM_ITERATIONS 300
#define METHB_TRESH 1

//implementation of SCA using shared memory
template<int MAX_ENDMEMBERS, int MAX_BLOCKDIM>
__global__ void SCA(float *AT, float *H, float *inputmua, float *x, float *mu, float *prevChrom, int prevMethbind, int currMethbind, int startind, int numBands, int numEndmembers, size_t pitch){
	#ifdef USE_SHMEM
	__shared__ float sh_x[MAX_ENDMEMBERS][MAX_BLOCKDIM];
	__shared__ float sh_mu[MAX_ENDMEMBERS][MAX_BLOCKDIM];
	#elif defined USE_LOCMEM
	float x_loc[MAX_ENDMEMBERS];
	float mu_loc[MAX_ENDMEMBERS];
	#endif

	//if (blockIdx.x == 5){
	//calculate initial values for mu and x
	int pixind = pitch*blockIdx.x + threadIdx.x;
	
	//should methb be included? (problematic chromophore, severe crosstalk)
	bool hasMethb = prevChrom[prevMethbind*gridDim.x*pitch + pixind] > METHB_TRESH;

	for (int i=0; i < numEndmembers; i++){
		float temp = 0;
		for (int j=0; j < numBands; j++){
			__shared__ float currAT;
			if (threadIdx.x == 0){
				currAT = AT[i*numBands + j];
			}
			__syncthreads();
			float inp = inputmua[pixind + gridDim.x*pitch*(j+startind)];
			temp += currAT*inp;
		}
		float xstart = 0;
		float mustart = ((i != currMethbind) || (hasMethb))*(-1*temp);//only initiate sh_mu to non-zero if methb is supposed to be present
		#ifdef USE_SHMEM
		sh_x[i][threadIdx.x] = xstart;
		sh_mu[i][threadIdx.x] = mustart;
		#elif defined USE_LOCMEM
		x_loc[i] = xstart;
		mu_loc[i] = mustart;
		#else
		int endind = pixind + i*gridDim.x*pitch;
		x[endind] = xstart;
		mu[endind] = mustart;
		#endif
	}
	__shared__ float sh_H[MAX_ENDMEMBERS][MAX_ENDMEMBERS];
	if (threadIdx.x == 0){
		for (int i=0; i < numEndmembers; i++){
			for (int j=0; j < numEndmembers; j++){
				sh_H[i][j] = H[i*numEndmembers + j];
			}
		}	
	}
	__syncthreads();


	//iterate
	for (int i=0; i < NUM_ITERATIONS; i++){
		for (int j=0; j < numEndmembers; j++){
			float xprev;
			float muprev;
			//extract previous values
			#ifdef USE_SHMEM
			xprev = sh_x[j][threadIdx.x];
			muprev = sh_mu[j][threadIdx.x];
			#elif defined USE_LOCMEM
			xprev = x_loc[j];
			muprev = mu_loc[j];
			#else
			int endind = pixind + j*gridDim.x*pitch;
			xprev = x[endind];
			muprev = mu[endind];
			#endif

			//calculate new value
			float temp = xprev - fdividef(muprev, sh_H[j][j]);
			temp = (temp > 0)*temp;
			
			//save new value to x
			#ifdef USE_SHMEM
			sh_x[j][threadIdx.x] = temp;
			#elif defined USE_LOCMEM
			x[j] = temp;
			#else
			x[endind] = temp;
			#endif

			//calculate new mu
			for (int k=0; k < numEndmembers; k++){
				float addition = (temp - xprev)*sh_H[k][j];
				#ifdef USE_SHMEM
				sh_mu[k][threadIdx.x] += addition;
				#elif defined USE_LOCMEM
				mu_loc[k] += addition;
				#else
				int endind = pixind + k*gridDim.x*pitch;
				mu[endind] += addition;
				#endif
			}
		}
	}
	#if (defined USE_SHMEM || defined USE_LOCMEM)
	for (int i=0; i < numEndmembers; i++){
		float val;
		#ifdef USE_SHMEM
		val = sh_x[i][threadIdx.x];
		#elif defined USE_LOCMEM
		val = x_loc[i];
		#endif
		x[pixind + i*gridDim.x*pitch] = val;
	}
	#endif
}


//implementation using registers
//YOLO
#define ATmultMua(Arg) i=Arg;\
		temp = 0;\
		for (int j=0; j < numBands; j++){\
			__shared__ float currAT;\
			if (threadIdx.x == 0){\
				currAT = AT[i*numBands + j];\
			}\
			__syncthreads();\
			float inp = inputmua[pixind + gridDim.x*pitch*(j+startind)];\
			temp += currAT*inp;\
		}\
		x##Arg = 0;\
		temp = ((i != currMethbind) || (hasMethb))*(temp);\
		mu##Arg = -1*temp
		/*mu_sh[Arg][threadIdx.x] = -1*temp*/

#define CalcX(Arg) j=Arg;\
		xprev = x##Arg;\
		muprev = mu##Arg;\
		/*muprev = mu_sh[Arg][threadIdx.x];*/\
		temp = xprev - fdividef(muprev, sh_H[j][j]);\
		temp = (temp > 0)*temp;\
		x##Arg = temp

#define CalcMu(Arg) k=Arg;\
		addition = (temp - xprev)*sh_H[k][j];\
		mu##Arg += addition
		/*mu_sh[Arg][threadIdx.x] += addition*/

#define updateMu CalcMu(0);\
		CalcMu(1);\
		if (MAX_ENDMEMBERS >= 3)\
			CalcMu(2);\
		if (MAX_ENDMEMBERS >= 4)\
			CalcMu(3);\
		if (MAX_ENDMEMBERS >= 5)\
			CalcMu(4);\
		if (MAX_ENDMEMBERS >= 6)\
			CalcMu(5);\
		if (MAX_ENDMEMBERS >= 7)\
			CalcMu(6)
		
//se opp for makrohelvete, dette ble skrevet for aa se om det ble noe raskere om jeg brukte av registrene istedet for shared memory men DESSVERRE var det ikke mulig aa opprette arrays i registrene uten like mye kuk som dette. ISRA-artikkelen lyver.  
#define MAX_BLOCKDIM_TEST 160
template<int MAX_ENDMEMBERS>
__global__ void SCAFast(float *AT, float *H, float *inputmua, float *x, float *mu, float *prevChrom, int prevMethbind, int currMethbind, int startind, int numBands, int numEndmembers, size_t pitch){
	//calculate initial values for mu and x
	int pixind = pitch*blockIdx.x + threadIdx.x;
	bool hasMethb = prevChrom[prevMethbind*gridDim.x*pitch + pixind] > METHB_TRESH;
	
	float temp;
	float x0, x1, x2, x3, x4, x5, x6;
	float mu0, mu1, mu2, mu3, mu4, mu5, mu6;
//	float mu_sh[MAX_ENDMEMBERS][MAX_BLOCKDIM_TEST];
	int i;

	//calculate initial coordinates
	ATmultMua(0);
	ATmultMua(1);
	if (MAX_ENDMEMBERS >= 3)
		ATmultMua(2);
	if (MAX_ENDMEMBERS >= 4)
		ATmultMua(3);
	if (MAX_ENDMEMBERS >= 5)
		ATmultMua(4);
	if (MAX_ENDMEMBERS >= 6)
		ATmultMua(5);
	if (MAX_ENDMEMBERS >= 7)
		ATmultMua(6);
	

	__shared__ float sh_H[MAX_ENDMEMBERS][MAX_ENDMEMBERS];
	if (threadIdx.x == 0){
		for (int i=0; i < numEndmembers; i++){
			for (int j=0; j < numEndmembers; j++){
				sh_H[i][j] = H[i*numEndmembers + j];
			}
		}	
	}
	__syncthreads();


	//iterate
	for (int i=0; i < NUM_ITERATIONS; i++){
		float xprev, muprev, temp, addition;
		int k,j;
		CalcX(0);
		updateMu;

		CalcX(1);
		updateMu;

		if (MAX_ENDMEMBERS >= 3){
			CalcX(2);
			updateMu;
		}

		if (MAX_ENDMEMBERS >= 4){
			CalcX(3);
			updateMu;
		}
		
		if (MAX_ENDMEMBERS >= 5){
			CalcX(4);
			updateMu;
		}
		
		if (MAX_ENDMEMBERS >= 6){
			CalcX(5);
			updateMu;
		}
	
		if (MAX_ENDMEMBERS >= 7){
			CalcX(6);
			updateMu;
		}	
	}
	
	x[pixind + 0*gridDim.x*pitch] = x0;
	x[pixind + 1*gridDim.x*pitch] = x1;
	if (MAX_ENDMEMBERS >= 3)
		x[pixind + 2*gridDim.x*pitch] = x2;
	if (MAX_ENDMEMBERS >= 4)
		x[pixind + 3*gridDim.x*pitch] = x3;
	if (MAX_ENDMEMBERS >= 5)
		x[pixind + 4*gridDim.x*pitch] = x4;
	if (MAX_ENDMEMBERS >= 6)
		x[pixind + 5*gridDim.x*pitch] = x5;
	if (MAX_ENDMEMBERS >= 7)
		x[pixind + 6*gridDim.x*pitch] = x6;
}
