RGBVisualize::RGBVisualize(int numBands, int numPixels, int numLine, vector<int> bands, QWidget *parent) : VisualizeStage(numBands, numPixels, numLine, bands, vector<float>(), vector<float>(), true, QStringList(), parent){
	
}

//some logic adapted from Norsk Elektro Optikk
void RGBVisualize::imageStream(float *blockData, int blockLine){
	int c2 = c;
	//get the data
	for(int i=0;i<blockLine;i++)
	for(int j=0;j<numPixels;j++)
	{
		float *vals = new float[bands.size()];
		for (int l=0; l < bands.size(); l++){
			vals[l] = blockData[i*numPixels*numBands + bands[l]*numPixels + j];
		}

		//RGB stuff
		imgDataRGB[c++] = sqrt(vals[0]);
		imgDataRGB[c++] = sqrt(vals[1]);
		imgDataRGB[c++] = sqrt(vals[2]);
		delete [] vals;
	}

	//push to imviews
	imView->newImageArray(imgDataRGB, numPixels, k);
	update();
	t=t+1;
	k=(blockLine)*t;

	if(k>=numLine)
	{
		int rp; 
		rp=numPixels*3*blockLine; // used to reduce repeated multiplication
		t=t-1;
		c=c-rp;
		//memmove(imgData,&imgData[rp],rp*t);
	}
}

FourTypeVisualize::FourTypeVisualize(int numPixels, int numLine, QWidget *parent) : VisualizeStage(1, numPixels, numLine, vector<int>(), vector<float>(), vector<float>(), true, QStringList(), parent){
	
}

//convert four values to a specific color
QColor fourvalToQColor(int val){
	QColor color;
	switch(val){
		case 0:
			color = QColor::fromRgb(255,216,0);
		break;
		case 1:
			color = QColor::fromRgb(0,126,255);
		break;
		case 2:
			color = QColor::fromRgb(0,217,71);
		break;
		case 3:
			color = QColor::fromRgb(255,0,0);
		break;
	}
	return color;
}

void FourTypeVisualize::imageStream(float *blockData, int blockLine){
	int c2 = c;
	//get the data
	for(int i=0;i<blockLine;i++)
	for(int j=0;j<numPixels;j++)
	{
		float val = blockData[i*numPixels*numBands + 0*numPixels + j];
		QColor color = fourvalToQColor(val);

		//RGB stuff
		imgDataRGB[c++] = color.red();
		imgDataRGB[c++] = color.green();
		imgDataRGB[c++] = color.blue();
	}

	//push to imviews
	imView->newImageArray(imgDataRGB, numPixels, k);
	update();
	t=t+1;
	k=(blockLine)*t;

	if(k>=numLine)
	{
		int rp; 
		rp=numPixels*3*blockLine; // used to reduce repeated multiplication
		t=t-1;
		c=c-rp;
		//memmove(imgData,&imgData[rp],rp*t);
	}
}

VisualizeStage::VisualizeStage(int numBands, int numPixels, int numLine, vector<int> bands, vector<float> maxVals, vector<float> minVals, bool RGBImage, QStringList winTitles, bool canShowSpectra, QWidget *parent) : QWidget(parent), numPixels(numPixels), numLine(numLine), bands(bands), numBands(numBands), RGBImage(RGBImage), maxVals(maxVals), minVals(minVals), canShowSpectra(canShowSpectra){
	firstTime = true;


	if ((bands.size() != maxVals.size()) || (maxVals.size() != minVals.size())){
		cerr << "Band, maxvals or minvals vectors not of the same size" << endl;
		exit(1);
	}

	//create arrays
	for (int i=0; i < bands.size(); i++){
		imgDataNonRGB.push_back(new uchar[numPixels*3*numLine]);
		floatDataNonRGB.push_back(new float[numPixels*3*numLine]);
	}
	imgDataRGB = new uchar[numPixels*3*numLine];
	floatDataRGB = new float[numPixels*3*numLine];
	k = 0;
	c = 0;
	t = 0;
	createPorts();
	QGridLayout *layout = new QGridLayout(this);

	//ready the image views
	imView = new ImageViewer;
	if (RGBImage){
		//layout->addWidget(imView, 1, 0);
		imView->show();
		imView->resize(450, 720);
		imView->setWindowTitle(this->QWidget::windowTitle());
	}
	for (int i=0; i < bands.size(); i++){
		ImageViewer *temp = new ImageViewer;
		if (winTitles.size() == bands.size()){
			temp->setWindowTitle(winTitles[i]);
		}
		temp->resize(450, 720);
		imViews.push_back(temp);
		if (!RGBImage){
			//layout->addWidget(temp, 1, i);
			temp->show();
		}
		cs.push_back(0);
	}
	cerr << "Canshowspectra: " << canShowSpectra << endl;
	if (canShowSpectra){
		cerr << "Hydata set to WHATEVER" << endl;
		hyData = new float[numPixels*numBands*numLine];
		imView->setHyData(hyData);
		connect(imView, SIGNAL(mouseClicked(int, int, ShouldSave)), imView, SLOT(forwardSimulation(int, int, ShouldSave)));
		PixelChooser *choosegoose = new PixelChooser;
		connect(choosegoose, SIGNAL(pretendMouseClicked(int, int, ShouldSave)), imView, SLOT(forwardSimulation(int, int, ShouldSave)));
		choosegoose->show();
	} else {
		cerr << "Hydata set to NULL" << endl;
		imView->setHyData(NULL);
	}
}
		
PixelChooser::PixelChooser(QWidget *parent) : QWidget(parent){
	pixel = new QLineEdit;
	line = new QLineEdit;
	QPushButton *ok = new QPushButton("Plot/save");
	connect(ok, SIGNAL(clicked()), this, SLOT(buttonTriggered()));
	QGridLayout *layout = new QGridLayout(this);
	layout->addWidget(pixel, 0, 0);
	layout->addWidget(line, 0, 1);
	layout->addWidget(ok, 0, 2);
}

void PixelChooser::buttonTriggered(){
	emit pretendMouseClicked(pixel->text().toInt(), line->text().toInt(), DO_SAVE_SPECTRUM);
}

void ImageViewer::newImageArray(uchar *imgData, int numPixels, int k){
	this->imgData = imgData;
	area->ensureVisible (numPixels, k);
	update(); 
	this->k = k; 
	this->numPixels = numPixels;
}
void ImageViewer::paintEvent(QPaintEvent *evt){
	QPoint point(0,0);
	QImage image((uchar*)imgData,numPixels,k,3*numPixels,QImage::Format_RGB888);

	int imHeight = image.height();
	int imWidth = image.width();

	QImage resImage = image.scaledToWidth(size().width()-30);

	int newHeight = resImage.height();
	int newWidth = resImage.width();

	widthScale = imWidth*1.0f/(newWidth*1.0f);
	heightScale = imHeight*1.0f/(newHeight*1.0f);

	QPainter painter(this);
	this->image->setPixmap(QPixmap::fromImage(resImage));
	this->image->adjustSize();
//	painter.drawImage(point,image.scaledToWidth(size().width()));
}

void VisualizeStage::connectThis(ImageViewer* v){
}
		
bool ImageViewer::eventFilter(QObject *object, QEvent *event){
	if (event->type() == QEvent::MouseMove){
		QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
		bar->showMessage("Pixel " + QString::number((int)(widthScale*mouseEvent->x())) + ", line " + QString::number((int)(heightScale*mouseEvent->y())));
	} else if ((event->type() == QEvent::MouseButtonPress)){
		QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
		int pixel = widthScale*mouseEvent->x();
		int line = heightScale*mouseEvent->y();
		emit mouseClicked(pixel, line, DO_NOT_SAVE_SPECTRUM);
	}
	return false;
}


void ImageViewer::forwardSimulation(int pixel, int line, ShouldSave shouldSave){
	string filename;
	forward(hyData, line, pixel, filename);
	if (shouldSave == DO_SAVE_SPECTRUM){
		filename = QFileDialog::getSaveFileName().toStdString();
		forward(hyData, line, pixel, filename); //tell GPU-DM to forward simulate this pixel
	}
};

//float to color
QColor convertFloatToColor(float w){
	//assume w lies between 0 and 1
	int b = (w <= 0.5)*w/0.5*255;
	int g = (w <= 0.75)*(w >= 0.25)*(w-0.25)/0.5*255;
	int r = (w >= 0.5)*(w <= 1)*(w-0.5)/0.5*255;
	if ((r <0) || (g < 0) || (b < 0)){
		r = 0;
		g = 0;
		b = 0;
	}
	return QColor::fromRgb(r,g,b);
}

//the indexing using the variable 'c' is adapted from Norsk Elektro Optikk
void VisualizeStage::imageStream(float* blockData, int blockLine){
	int c2 = c;

	//get the data
	for(int i=0;i<blockLine;i++){
		for(int j=0;j<numPixels;j++){
			float *vals = new float[bands.size()];
			for (int l=0; l < bands.size(); l++){
				if (bands[l] >= 0){
					vals[l] = blockData[i*numPixels*numBands + bands[l]*numPixels + j];
					
					//convert to something that lies within 0 and 1 within the allowed range of values
					float a = 1.0f/(maxVals[l] - minVals[l]);
					float b = -a*minVals[l];

					//convert
					float val = (vals[l]*a + b)*255.0f;
					if (val > 255){
						val = 255;
					} else if (val < 0){
						val = 0;
					}
					vals[l] = val;
					if (!RGBImage){
						val /= 255.0f;
						QColor rgb = convertFloatToColor(val);
						imgDataNonRGB[l][cs[l]++] = rgb.red();
						imgDataNonRGB[l][cs[l]++] = rgb.green();
						imgDataNonRGB[l][cs[l]++] = rgb.blue();
					} else {
						imgDataNonRGB[l][cs[l]++] = val;
						imgDataNonRGB[l][cs[l]++] = val;
						imgDataNonRGB[l][cs[l]++] = val;
					}
				} else {
					vals[l] = 0.0f;
				}
			}

			//RGB stuff
			imgDataRGB[c++] = vals[0];
			if (bands.size() > 2){
				imgDataRGB[c++] = vals[2];
			} else {
				imgDataRGB[c++] = 0;
			}
			if (bands.size() > 1){
				imgDataRGB[c++] = vals[1];
			}
			delete [] vals;
		}
	}	

	//push to imviews
	imView->newImageArray(imgDataRGB, numPixels, k);
	for (int i=0; i < bands.size(); i++){
		imViews[i]->newImageArray(imgDataNonRGB[i], numPixels, k);
	}
	update();
	t=t+1;
	k=(blockLine)*t;


	//adapted from norsk elektro optikk
	if(k>=numLine){
		int rp; 
		rp=numPixels*3*blockLine; 
		t=t-1;
		c=c-rp;
	}


}


void VisualizeStage::execute(){
	//omitted
}


