enum ShouldSave{DO_SAVE_SPECTRUM, DO_NOT_SAVE_SPECTRUM};

//image viewer. Parts of this is adapted from norsk elektro optikk
//for displaying physical images on input
class ImageViewer : public QWidget{
	Q_OBJECT
	public:
		ImageViewer(QWidget *parent = NULL) : QWidget(parent){
			QGridLayout *lay = new QGridLayout(this);
			area = new QScrollArea;
			lay->addWidget(area);
			image = new QLabel;
			image->setBackgroundRole(QPalette::Base);
		        image->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
			area->setWidget(image);
			image->setScaledContents(true);
			bar = new QStatusBar;
			lay->addWidget(bar);
			bar->showMessage("Ready");
			image->setMouseTracking(true);
			image->installEventFilter(this);
			hyData = NULL;

		};
		void newImageArray(uchar *imgData, int numPixels, int k); //set QImage to incoming image array
		void setHyData(float *hyData){this->hyData = hyData;}; //save full hyperspectral data in the image viewer it is set as the "main" image viewer to catch all mouse click events and send data to invertReflectance
	protected:
		void paintEvent(QPaintEvent *evt);
		bool eventFilter(QObject *object, QEvent *event); //event filter for catching mouse hover events and displaying pixel placements at bottom
	private:
		QStatusBar *bar;
		QScrollArea *area;
		QLabel *image;
		uchar *imgData;
		int numPixels, k;
		float heightScale;
		float widthScale;
		float *hyData;
	signals:
		void mouseClicked(int, int, ShouldSave);	
	public slots:
		void forwardSimulation(int, int, ShouldSave);

};

//for inputting pixel number and line number and inverting the specific pixel
class PixelChooser : public QWidget{
	Q_OBJECT
	public:
		PixelChooser(QWidget *parent = NULL);
	private:
		QLineEdit *line;
		QLineEdit *pixel;
	private slots:
		void buttonTriggered();
	signals:
		void pretendMouseClicked(int pixel, int line, ShouldSave);
};


//adapted from norsk elektro optikk
//stage for viewing chromophore images
//images are only sent to one or several imageviewer classes
class VisualizeStage{ //inheritance classes are omitted
	public:
		VisualizeStage(int numBands, int numPixels, int numLine, std::vector<int> bands, std::vector<float> maxVals, std::vector<float> minVals, bool RGBimage, QStringList winTitles, bool canShowSpectra = false, QWidget *parent = NULL); //canshowspectra: for saving all hyperspectral data to the image viewer. RGBImage: if set, will mix the first bands to an RGB image. Otherwise display monochromatic images
		void setWindowTitle(QString title){imView->setWindowTitle(title);};

		//framework
		//
		// omitted
		//
	protected:
		 bool canShowSpectra;

		//indices for navigating the image
		int k;
		int c;
		int t;

		//image properties
		int numBands, numPixels, interleave;

		//received data
		quint16* blockData;

		//current line
		int numLine;

		//saved image data
		uchar* imgDataRGB; //RGB
		float* floatDataRGB; //grayscale
		
		std::vector<uchar*> imgDataNonRGB; //the data that is to be displayed as RGB images
		std::vector<float*> floatDataNonRGB; //data to be displayed as grayscale

		//indices navigating the received images
		std::vector<int> cs;

		//framework
		//
		// omitted
		//
		
		bool RGBImage; //whether it is supposed to be an RGB image or not
		
		std::vector<int> bands; //list of bands to be displayed (or chromophores or whatever)
		std::vector<ImageViewer*> imViews; //grayscale image views
		std::vector<float> maxVals; //list of maximum values
		std::vector<float> minVals; //list of minimum values

		bool firstTime; //whether the line is the first line received
		
		virtual void imageStream(float* blockData, int blockLine); //inputting data to image
		ImageViewer *imView; //RGB image view

		float *hyData; //hyperspectral data, incoming data is saved if canShowSpectra is set
};

//subclass of VisualizeStage for converting uncalibrated hyperspectral images to RGB images
class RGBVisualize : public VisualizeStage{
	public:
		RGBVisualize(int numBands, int numPixels, int numLine, std::vector<int> bands, QWidget *parent = NULL);
	protected:
		virtual void imageStream(float *blockData, int blockLine);
};

//set 0, 1, 2, 3 in an incoming array to different colors. 
class FourTypeVisualize : public VisualizeStage{
	public:
		FourTypeVisualize(int numPixels, int numLine, QWidget *parent = NULL);
	protected:
		virtual void imageStream(float *blockData, int blockLine);
};
